﻿#==============================================================================
# ★ RGSS3-Extension
# LNX11a_XPスタイルバトル preview ver.
#
# 基本的な使いかた：スクリプトを入れるだけで動きます。
#
# ・アクターのバトラーグラフィックを顔グラフィック以外にしたい場合は、
# 　アクターのメモに
# 　　バトラーグラフィック = "ファイル名"
# 　と記述します。
# 　ファイルは Graphics/Battler/ フォルダから読み込みます。
#
# 　version   : 0.11 (12/01/15)
#
#==============================================================================

module LNX11
  #
  #
  # 　これらの一見複雑そうな設定項目は必ずしも編集する必要はありません。
  # スクリプトの扱いに慣れてきて、デザインをアレンジしてみたいと思ったら
  # 少しずつ編集してみてください。
  #
  #
  #-● 設定項目の目次 -------------------------------------------------------
  # line 038~ : 基本設定
  # line 063~ : アクターの背景
  # line 080~ : メッセージ表示
  # line 116~ : アクターのバトラーグラフィック
  # line 141~ : バトルステータス
  # line 172~ : アクターコマンド
  # line 205~ : パーティコマンド
  # line 218~ : ターゲットカーソル
  # line 231~ : ターゲットヘルプ
  # line 243~ : ダメージポップアップ
  # line 441~ : その他
  #--------------------------------------------------------------------------
  # ● 基本設定
  #--------------------------------------------------------------------------
  # デフォルトで表示するアクターのバトラーグラフィック
  # 　0 = 顔グラフィック / 1 = 歩行グラフィック
  DEFAULT_BATTLER_GRAPHIC = 0  # 規定値:0
  
    #------------------------------------------------------------------------
    # ○ 0:顔グラフィック の設定
    #------------------------------------------------------------------------
    # 顔グラフィックの背景グラデーション色([Color1, Color2]) 
    FACE_BACK_COLOR = [Color.new(0, 0, 0, 0), Color.new(0, 0, 0, 0)]
             # 規定値:[Color.new(0, 0, 0, 0), Color.new(0, 0, 0, 0)]
             
  # 画面の色調変更をアクターに適用する true = 有効 / false = 無効
  # * true にするとアクターの背景(ウィンドウ)にも適用されます
  ACTOR_SCREEN_TONE       = false  # 規定値:false
  
  # TP のポップアップ設定
  # 　0 = すべてポップアップする / 1 = アクターのみ / 2 = ポップアップしない
  TP_POPUP_TYPE           = 1      # 規定値:1
  
  # アイコン無しのステートをポップアップしない true = 有効 / false = 無効
  INDEXZERO_NO_POPUP      = true   # 規定値:true
  
  #--------------------------------------------------------------------------
  # ● 設定:アクターの背景
  #--------------------------------------------------------------------------
  # アクターの背景のタイプ
  # 　0 = 無し / 1 = グラデーション / 2 = ウィンドウ
  ACTOR_BACKGROUND        = 2   # 規定値:2
  
  # アクター背景の高さ
  ACTOR_BACKGROUND_HEIGHT = 112 # 規定値:112
  
    #------------------------------------------------------------------------
    # ○ タイプ 1:グラデーション の設定
    #------------------------------------------------------------------------
    # グラデーションの色([Color1, Color2])
    ACTOR_BG_GRADIENT_COLOR = [Color.new(0, 0, 0, 0), Color.new(0, 0, 0, 224)]
                     # 規定値:[Color.new(0, 0, 0, 0), Color.new(0, 0, 0, 224)]
      
  #--------------------------------------------------------------------------
  # ● 設定:メッセージ表示
  #--------------------------------------------------------------------------
  # メッセージウィンドウの位置 0 = 上 / 1 = 中 / 2 = 下
  MESSAGE_WINDOW_POSITION = 0 # 規定値:0
  
  # バトルログのタイプ
  # 0 = VXAceデフォルト / 1 = 蓄積型(オリジナル) / 2 = バトルログ無し
  BATTLELOG_TYPE          = 1 # 規定値:1
  
    #------------------------------------------------------------------------
    # ○ バトルログのタイプ 1:蓄積型(オリジナル) の設定
    #------------------------------------------------------------------------
    # 一度に表示する行数
    STORAGE_LINE_NUMBER     = 6     # 規定値:6

    # 一行の高さ
    STORAGE_LINE_HEIGHT     = 18    # 規定値:18
    
    # 自動的にログが進行する時間(フレーム)
    STORAGE_UP_MESSAGE_TIME = 90    # 規定値:90
    
    # ターン終了時にバトルログを消去する true = 有効 / false = 無効
    STORAGE_TURNEND_CLEAR   = true  # 規定値:true
    
    # 座標の調整 :x = X座標 / :y = Y座標
    STORAGE_OFFSET = {:x => 0, :y => 6} 
            # 規定値:{:x => 0, :y => 6}
    
    # フォント :size = フォントサイズ / :out_color = テキストの縁取り色
    STORAGE_FONT   = {:size => 20, :out_color => Color.new(0, 0, 0, 192)}
            # 規定値:{:size => 20, :out_color => Color.new(0, 0, 0, 192)}
    
    # 背景のグラデーションの色([Color1, Color2])
    STORAGE_GRADIENT_COLOR = [Color.new(0, 0, 0, 128), Color.new(0, 0, 0, 0)]
  
  #--------------------------------------------------------------------------
  # ● 設定:アクターのバトラーグラフィック
  #--------------------------------------------------------------------------
  # アクター表示を中央揃えにする true = 有効 / false = 左揃え
  ACTOR_CENTERING = true   # 規定値:true

    #------------------------------------------------------------------------
    # ○ アクター表示中央揃えの設定
    #------------------------------------------------------------------------
    # アクター間のスペースの大きさ補正
    ACTOR_SPACING_ADJUST = 32 # 規定値:32
    
  # 座標の調整 :x = X座標 / :y = Y座標
  # * アクターコマンドのX座標には影響しません。
  ACTOR_OFFSET    = {:x => -16, :y => 0}
            #規定値:{:x => -16, :y => 0}
  
  # アクター表示領域と画面端との余白 :side = 左右の余白 / :bottom = 下の余白
  ACTOR_PADDING   = {:side => 4, :bottom =>  8}
           # 規定値:{:side => 4, :bottom =>  8}
  
  # アクターに対する画面アニメーションのY座標修正
  # 　位置が[画面]のアニメーションをパーティの中心に表示されるようにします。
  SCREEN_ANIMATION_OFFSET = 128 # 規定値:128
  
  #--------------------------------------------------------------------------
  # ● 設定:バトルステータス
  #--------------------------------------------------------------------------
  # 座標の調整  :x = X座標 / :y = Y座標
  #　 X 座標はアクターの中心を基準とします。
  # * パーティメンバーの数やステータス幅によっては、X 座標が自動的に
  # 調整される場合があります。
  STATUS_OFFSET = {:x => 64, :y => -12}
         # 規定値:{:x => 64, :y => -12}
  
  # 画面左右端の余白
  STATUS_SIDE_PADDING   = 6     # 規定値:6
  
  # アクター1人あたりのステータス幅
  STATUS_WIDTH          = 72    # 規定値:72

  # ステータス幅の自動調整を行う
  STATUS_AUTOADJUST     = true  # 規定値:true
  
  # 行の高さ
  STATUS_LINE_HEIGHT    = 22    # 規定値:22
  
  # アクターの名前のフォントサイズ 値を 0 にすると非表示になる
  STATUS_NAME_SIZE      = 22    # 規定値:22
  
  # HP/MP/TPのフォントサイズ
  STATUS_PARAM_SIZE     = 22    # 規定値:22
  
  # HP/MP/TPゲージの透明度 (0-255)
  STATUS_GAUGE_OPACITY  = 192   # 規定値:192

  #--------------------------------------------------------------------------
  # ● 設定:アクターコマンド
  #--------------------------------------------------------------------------
  # コマンドの項目数が 4 を超える場合、項目がすべて見えるように
  # コマンドウィンドウの高さを拡張する true = 有効 / false = 無効
  ACTOR_COMMAND_NOSCROLL   = true  # 規定値:true
  
  # コマンドを縦並びにする true = 縦並び / false = 横並び
  ACTOR_COMMAND_VERTICAL   = true  # 規定値:true

  # 文字揃え 0 = 左揃え / 1 = 中央揃え / 2 = 右揃え
  ACTOR_COMMAND_ALIGNMENT  = 0     # 規定値:0
  
  # ウィンドウの横幅
  # 　コマンドを横並びにする場合、大きめの値を設定してください。
  ACTOR_COMMAND_WIDTH      = 128   # 規定値:128
  
  # ウィンドウの位置
  # 　0 = アクターの頭上 / 1 = Y座標を固定 / 2 = XY座標を固定
  ACTOR_COMMAND_POSITION   = 0     # 規定値:0

    #------------------------------------------------------------------------
    # ○ ウィンドウの位置 1:Y座標固定 2:XY座標 の設定
    #------------------------------------------------------------------------
    # 固定Y座標の基準位置
    # 0 = 画面下端 - ウィンドウの高さ / 画面上端
    ACTOR_COMMAND_Y_POSITION = 0   # 規定値:0
  
    # ウィンドウのXY座標修正 :x = X座標 / :y = Y座標
    # * コマンド位置が 1 or 2 の場合、ここで設定した値で座標を固定します。
    ACTOR_COMMAND_OFFSET = {:x => 0, :y => -16}
                  # 規定値:{:x => 0, :y => -16}
  
  #--------------------------------------------------------------------------
  # ● 設定:パーティコマンド
  #--------------------------------------------------------------------------
  # コマンドを縦並びにする true = 縦並び / false = 横並び
  PARTY_COMMAND_VERTICAL  = false # 規定値:false
  
  # 文字揃え 0 = 左揃え / 1 = 中央揃え / 2 = 右揃え
  PARTY_COMMAND_ALIGNMENT = 1     # 規定値:1

  # ウィンドウの座標 :x = X座標 / :y = Y座標 / :width = 幅
  PARTY_COMMAND_RECT      = {:x => 0, :y => 0, :width => Graphics.width}
                   # 規定値:{:x => 0, :y => 0, :width => Graphics.width}

  #--------------------------------------------------------------------------
  # ● 設定:ターゲットカーソル
  #--------------------------------------------------------------------------
  # カーソルグラフィックのファイル名 (Graphics/System/*)
  # 　"" を設定した場合、スクリプトがカーソルを自動生成します。
  CURSOR_NAME       = "" # 規定値:""
 
  # アニメーションスピード
  CURSOR_ANI_SPEED  = 3  # 規定値:3
  
  # 移動スピード
  CURSOR_SPEED      = 3  # 規定値:3

  #--------------------------------------------------------------------------
  # ● 設定:ターゲットヘルプ
  #--------------------------------------------------------------------------
  # バトラーの各パラメータ表示 true = 表示 / false = 非表示
  HELP_ACTOR_PARAM = {:hp => true ,:mp => true ,:tp => true ,:state => true }
  HELP_ENEMY_PARAM = {:hp => false,:mp => false,:tp => false,:state => true }
    # アクター規定値:{:hp => true ,:mp => true ,:tp => true ,:state => true }
    # 敵キャラ規定値:{:hp => false,:mp => false,:tp => false,:state => true } 
    
  # HP/MP/TP ゲージの幅
  HELP_PARAM_WIDTH = 72  # 規定値:72
  
  #--------------------------------------------------------------------------
  # ● 設定:ダメージポップアップ
  #--------------------------------------------------------------------------  
  # ポップアップ位置 0 = 足元 / 1 = 中心 / 2 = 頭上 / 3 = 一定(アクターのみ)
  ACTOR_POPUP_POSITION   = 3  # アクター     規定値:3
  ENEMY_POPUP_POSITION   = 1  # 敵キャラ     規定値:1
  LEVELUP_POPUP_POSITION = 3  # レベルアップ 規定値:3
  
    #------------------------------------------------------------------------
    # ○ ポップアップ位置 3:一定(アクターのみ) の設定
    #------------------------------------------------------------------------
    # アクターのポップアップの Y 座標
    # 　ゲーム画面の高さ(Graphics.height)を基準とします。
    ACTOR_POPUP_Y     =  -100  # 規定値:-100
    
    # レベルアップのポップアップの Y 座標
    # 　ゲーム画面の高さ(Graphics.height)を基準とします。
    LEVELUP_POPUP_Y   =  -112  # 規定値:-112
  
  # ステート等のテキストのフォント
  TEXT_FONT     =  ["Arial Black", "VL Gothic"] # ["Arial Black", "VL Gothic"] 
  TEXT_FONT_J   =  ["VL Gothic"] # 日本語用。規定値:["VL Gothic"]
  
  # 数字のフォント ([第一候補, 第二候補, ..])
  NUMBER_FONT   =  ["Arial Black", "VL Gothic"] # ["Arial Black", "VL Gothic"] 
  
  # 数字のサイズ・字間調整
  # 　:fontsize = フォントサイズ / :spacing = 字間
  # 　:width    = 一字あたりの幅 / :height = 一字あたりの高さ
  LARGE_NUMBER  =  {:fontsize => 36, :spacing => -8,
                    :width    => 26, :height  => 32 }
  SMALL_NUMBER  =  {:fontsize => 28, :spacing => -8,
                    :width    => 22, :height  => 24 }
  # 大サイズ規定値:{:fontsize => 36, :spacing => -8,
  #                 :width    => 26, :height  => 32 }
  # 小サイズ規定値:{:fontsize => 36, :spacing => -8,
  #                 :width    => 26, :height  => 32 }
  
  # テキストのフォントサイズ倍率
  # :normal   = テキスト単体のテキスト
  # :pos_top  = ダメージ値の上部に表示される装飾テキスト
  # :pos_left = ダメージ値の左側に表示される装飾テキスト
  # * 数字のフォントサイズからの割合で指定します。
  TEXT_SIZERATE = {:normal => 0.8, :pos_top => 0.6, :pos_left => 0.75}
         # 規定値:{:normal => 0.8, :pos_top => 0.6, :pos_left => 0.75}
  
  # ダメージポップアップの文字装飾 [テキスト, 修飾文字の位置(0 = 上 / 1 = 左)]
  # * 文字列中の [MP] [TP] はデータベースで設定した用語(短)に置き換わります
  DECORATION_NUMBER = {            # 規定値
  :critical    => ["CRITICAL", 0], # ["CRITICAL", 0] / クリティカル
  :weakness    => ["WEAKNESS", 0], # ["WEAKNESS", 0] / 弱点ダメージ
  :resist      => ["RESIST"  , 0], # ["RESIST"  , 0] / 耐性ダメージ
  :mp_damage   => ["[MP]"    , 0], # ["[MP]"    , 0] / MP ダメージ・回復
  :mp_plus     => ["[MP]+"   , 1], # ["[MP]+"   , 1] / MP 再生
  :mp_minus    => ["[MP]-"   , 1], # ["[MP]-"   , 1] / MP 消費
  :tp_plus     => ["[TP]+"   , 1], # ["[TP]+"   , 1] / TP 増加
  :tp_minus    => ["[TP]-"   , 1], # ["[TP]-"   , 1] / TP 減少
  }
  # ステートや能力強化/弱体のポップアップの文字装飾
  # 　%s にステート・能力名が入る
  DECORATION_TEXT = {              # 規定値
  :add_state   => "+%s",           # "+%s"      / ステート付加
  :rem_state   => "-%s",           # "-%s"      / ステート解除
  :add_buff    => "%s UP",         # "%s UP"    / 能力強化付加
  :add_debuff  => "%s DOWN",       # "%s DOWN"  / 能力弱体付加
  :rem_buff    => "-%s Buff",      # "-%s Buff" / 能力強化/弱体解除
  }
  # ポップアップの表示用語
  POPUP_VOCAB = {                  # 規定値
  :miss        => "MISS!",         # "MISS!"      / ミス
  :counter     => "Counter",       # "Counter"    / 反撃
  :reflection  => "Reflection",    # "Reflection" / 反射
  :substitute  => "Substitute",    # "Substitute" / 身代わり
  :levelup     => "LEVELUP!",      # "LEVELUP!"   / レベルアップ
  }
  # 能力強化/弱体ポップアップのパラメータ表記
  # * 文字列中の [HP] [MP] はデータベースで設定した用語(短)に置き換わります
  POPUP_VOCAB_PARAMS = [ # 規定値
  "MAX[HP]",             # "MAX[HP]" / 最大HP   MAXimum Hit Point
  "MAX[MP]",             # "MAX[MP]" / 最大MP   MAXimum Magic Point
  "ATK",                 # "ATK"     / 攻撃力   ATtacK power
  "DEF",                 # "MATK"    / 防御力   DEFense power
  "MATK",                # "MDEF"    / 魔法力   Magic ATtacK power
  "MDEF",                # "MDEF"    / 魔法防御 Magic DEFense power
  "AGI",                 # "AGI"     / 敏捷性   AGIlity
  "LUCK",                # "LUCK"    / 運       LUCK
  ]
  # ダメージポップアップの色([Color1, Color2])
  # * ハッシュの順序を変更しないこと
  POPUP_COLOR = {
  # HP ダメージ(ミス)
  :hp_damage     => [Color.new(255, 255, 255), Color.new(  0,   0,   0)],
  # クリティカルダメージ
  :critical      => [Color.new(255, 255,  80), Color.new(224,  32,   0)],
  # 弱点ダメージ
  :weakness      => [Color.new(255, 255, 255), Color.new(  0,  56, 144)],
  # 耐性ダメージ
  :resist        => [Color.new(232, 224, 216), Color.new( 56,  48,  40)],
  # HP 回復(再生,吸収)
  :hp_recovery   => [Color.new( 96, 255, 128), Color.new(  0,  64,  32)],
  # MP ダメージ
  :mp_damage     => [Color.new(248,  80, 172), Color.new( 48,   0,  32)],
  # MP 回復(再生,吸収)
  :mp_recovery   => [Color.new(160, 240, 255), Color.new( 32,  48, 144)],
  # TP 増減
  :tp_damage     => [Color.new(248, 240,  64), Color.new(  0,  80,  40)],
  # 有利なステートの付加
  :add_state     => [Color.new(255, 255, 255), Color.new(  0,  56, 144)],
  # 有利なステートの解除
  :rem_state     => [Color.new(224, 232, 240), Color.new( 32,  64, 128, 128)],
  # ステートの付加
  :add_badstate  => [Color.new(255, 255, 255), Color.new(  0,   0,   0)],
  # ステートの解除
  :rem_badstate  => [Color.new(224, 224, 224), Color.new( 32,  32,  32, 128)],
  # 能力強化の付加
  :add_buff      => [Color.new(255, 255, 192), Color.new( 96,  64,   0)],
  # 能力弱体の付加
  :add_debuff    => [Color.new(200, 224, 232), Color.new( 40,  48,  56)],
  # 能力強化/弱体の解除
  :rem_buff      => [Color.new(224, 224, 224), Color.new( 32,  32,  32, 128)],
  # 反撃
  :counter       => [Color.new(255, 255, 224), Color.new(128,  96,   0)],
  # 反射
  :reflection    => [Color.new(224, 255, 255), Color.new(  0,  96, 128)],
  # 身代わり
  :substitute    => [Color.new(224, 255, 224), Color.new(  0, 128,  64)],
  # レベルアップ
  :levelup       => [Color.new(255, 255, 255), Color.new(  0,   0,   0)],
  }
  # ポップアップのタイプ
  #  0 = 通常のポップアップ  / 1 = 小さいポップアップ
  #  2 = ゆっくり上昇(大)    / 3 = ゆっくり上昇(小)
  #  4 = スライド(大)        / 5 = スライド(小)
  #  6 = オーバーレイ(大)    / 7 = オーバーレイ(小)
  # -1 = ポップアップしない
  POPUP_TYPE = {         # 規定値
  :miss          =>  0,  #  0 / ミス
  :hp_damage     =>  0,  #  0 / HP ダメージ
  :hp_slipdamage =>  1,  #  1 / HP スリップダメージ
  :hp_recovery   =>  0,  #  0 / HP 回復
  :hp_regenerate =>  1,  #  1 / HP 再生
  :hp_drain      =>  0,  #  0 / HP 被吸収
  :hp_drainrecv  =>  0,  #  0 / HP 吸収回復
  :mp_damage     =>  0,  #  0 / MP ダメージ
  :mp_slipdamage =>  1,  #  1 / MP スリップダメージ
  :mp_recovery   =>  0,  #  0 / MP 回復
  :mp_regenerate =>  1,  #  1 / MP 再生
  :mp_drain      =>  0,  #  0 / MP 被吸収
  :mp_drainrecv  =>  0,  #  0 / MP 吸収回復
  :mp_paycost    => -1,  # -1 / MP 消費
  :tp_damage     =>  1,  #  1 / TP 増加(ダメージ)
  :tp_charge     => -1,  # -1 / TP チャージ(被ダメージ時の TP 増加)
  :tp_gain       =>  7,  #  7 / TP 得
  :tp_regenerate =>  1,  #  1 / TP 再生(スリップダメージ)
  :tp_paycost    => -1,  # -1 / TP 消費
  # ステート/能力
  :add_state     =>  2,  #  2 / 有利なステートの付加
  :rem_state     =>  2,  #  2 / 有利なステートの解除
  :add_badstate  =>  2,  #  2 / 不利なステートの付加
  :rem_badstate  =>  2,  #  2 / 不利なステートの解除
  :add_buff      =>  3,  #  3 / 能力強化の付加
  :add_debuff    =>  3,  #  3 / 能力弱体の付加
  :rem_buff      =>  3,  #  3 / 能力強化/弱体の解除
  # 特殊
  :counter       =>  6,  #  6 / 反撃
  :reflection    =>  6,  #  6 / 反射
  :substitute    =>  5,  #  5 / 身代わり
  }
  # 0:跳ねるポップアップの動き(大)
  LARGE_MOVEMENT = {
  :inirate    => 6.4, :gravity      => 0.68, :side_scatter => 1.2,
  :ref_height =>  32, :ref_factor   => 0.60, :ref_count    =>   2,
  :duration   =>  40, :fadeout      =>   20
  }
  # 1:跳ねるポップアップの動き(小)
  SMALL_MOVEMENT = {
  :inirate    => 4.4, :gravity      =>  0.6, :side_scatter => 0.0,
  :ref_height =>  12, :ref_factor   =>  0.7, :ref_count    =>   0,
  :duration   =>  60, :fadeout      =>   16
  }
  # 2,3:上昇ポップアップの動き
  RISE_MOVEMENT    = {:rising_speed => 0.75, :line_spacing => 0.7,
                      :duration     =>   40, :fadeout      =>   8 }
  
  # 4,5:スライドポップアップの動き
  SLIDE_MOVEMENT   = {:x_speed      =>    2, :line_spacing => 0.7,
                      :duration     =>   50, :fadeout      =>  32 }
  
  # 6,7:オーバーレイポップアップの動き
  OVERLAY_MOVEMENT = {:duration     =>   36, :fadeout      =>  32 }

  # :inirate      = 初速度               / :gravity    = 重力加速度
  # :side_scatter = 左右移動のばらつき   / :ref_height = 跳ね返り位置
  # :ref_factor   = 反発係数             / :ref_count  = 跳ね返り回数
  # :duration     = 消え始めるまでの時間 / :fadeout    = 消える速度
  # :rising_speed = 上昇速度             / :x_speed    = 横移動速度
  # :line_spacing = ポップアップの行間
  
  #--------------------------------------------------------------------------
  # ● 設定:その他
  #--------------------------------------------------------------------------
  # 2ターン目からパーティコマンド入力を省略する(Bボタンで再入力できる)
  PARTY_COMMAND_SKIP = true   # 規定値:true

  # アクターコマンドのカーソル位置を記憶する
  LAST_ACTOR_COMMAND = true   # 規定値:true 
  
  # 対象選択のカーソル位置を記憶する
  LAST_TARGET        = true   # 規定値:true
  
  # スキルリスト・アイテムリストを項目数に合わせてリサイズする
  # 　true = 有効 / false = 無効
  FITTING_LIST       = true   # 規定値:true
  
  # 行動時の白フラッシュを強めにする true = 有効 / false = 無効
  ENHANCE_WHITEN     = true   # 規定値:true
  
  # アクターがダメージを受けた時、画面を揺らす true = 有効 / false = 無効
  ACTOR_DAMAGE_SHAKE = false  # 規定値:false
  
  # 敵キャラのX座標を画面サイズに合わせて修正する true = 有効 / false = 無効
  TROOP_X_SCREEN_FIX = true   # 規定値:true
  
  # 敵キャラのY座標の調整
  TROOP_Y_OFFSET     = 0      # 規定値:0
  
  # 顔グラフィックのサイズ :width = 幅 / :height = 高さ
  # * スクリプトを編集して顔グラフィックのサイズを
  # 　変更している場合のみ、この値を書き換えてください。
  FACE_SIZE          = {:width => 96, :height => 96}
              # 規定値:{:width => 96, :height => 96}

  #
  # 設定ここまで お疲れ様でした。
  #
  #--------------------------------------------------------------------------
  # ● 正規表現
  #--------------------------------------------------------------------------
  # アクター：バトラーグラフィック = "ファイル名"
  RE_BATTLER = /(?:バトラーグラフィック|battler_graphic)\s*=\s*"([^"]*)"/i
  # アイテム/スキル：使用時アニメ = アニメーションID
  RE_USE_ANIMATION = /(?:使用時アニメ|use_animation)\s*=\s*(\d+)/i
  # ステート：ステートアニメ = アニメーションID
  RE_STATE_ANIMATION = /(?:ステートアニメ|state_animation)\s*=\s*(\d+)/i
  # ステート：ポップアップ表示名 = "表示名"
  RE_STATE_DISPLAY = /(?:ポップアップ表示名|display_name)\s*=\s*"([^"]*)"/i
  # ステート：ポップアップ非表示
  RE_STATE_NO_DISPLAY = /(?:ポップアップ非表示|no_display)/i
  # ステート：付加ポップアップ非表示
  RE_STATE_ADD_NO_DISPLAY = /(?:付加ポップアップ非表示|add_no_display)/i
  # ステート：解除ポップアップ非表示
  RE_STATE_REM_NO_DISPLAY = /(?:解除ポップアップ非表示|remove_no_display)/i
  # ステート：有利なステート
  RE_STATE_ADVANTAGE = /(?:有利なステート|advantage_state)/i
  # ステート：付加ポップアップタイプ = 0..5
  RE_STATE_ADD_TYPE = /(?:付加ポップアップ|add_popup_type)\s*=\s*([0-5]+)/i
  # ステート：解除ポップアップタイプ = 0..5
  RE_STATE_REM_TYPE = /(?:解除ポップアップ|remove_popup_type)\s*=\s*([0-5]+)/i
  # ステート：修飾文字非表示
  RE_STATE_NO_DECORATION = /(?:修飾文字非表示|no_decoration)/i
  # ステート：付加修飾文字非表示
  RE_STATE_ADD_NO_DECORATION = /(?:付加修飾文字非表示|add_no_decoration)/i
  # ステート：解除修飾文字非表示
  RE_STATE_REM_NO_DECORATION = /(?:解除修飾文字非表示|remove_no_decoration)/i
  #--------------------------------------------------------------------------
  # ● バトラーグラフィックのスクリプト指定
  #--------------------------------------------------------------------------
  # LNX11.バトラーグラフィック(id ,filename)
  def self.battler_graphic(id ,filename)
    if id.is_a?(Numeric) && filename.is_a?(String)
      p "LNX11a:バトラーグラフィックを変更しました:ID#{id} #{filename}"
      $game_actors[id].battler_graphic_name = filename
    else
      errormes = "LNX11a:バトラーグラフィック指定の引数が正しくありません。"
      p errormes, "LNX11a:バトラーグラフィックの指定は行われませんでした。"
      msgbox errormes
    end
  end
  def self.バトラーグラフィック(id ,filename)
    self.battler_graphic(id ,filename)
  end
end

#==============================================================================
# ★ RGSS3-Extension
# LNX09_アニメーション速度変更
#==============================================================================

module LNX09
  #--------------------------------------------------------------------------
  # ● 基本設定
  #--------------------------------------------------------------------------
  # マップ画面のデフォルトのアニメーション速度 (1以上)
  DEFAULT_MAP_SPEED_RATE    = 4  # 規定値:4

  # バトル中のデフォルトのアニメーション速度 (1以上)
  DEFAULT_BATTLE_SPEED_RATE = 3  # 規定値:3
  
  #--------------------------------------------------------------------------
  # ● 正規表現
  #--------------------------------------------------------------------------
  RE_ANISPEED = /\[(\d+)\]/
end

#==============================================================================
# ■ RPG::Animation
#------------------------------------------------------------------------------
# 　アニメーションのデータクラス。
#==============================================================================

class RPG::Animation
  #------------------------------------------------------------------------
  # ● [追加]:名前からアニメーション再生速度を取得
  #------------------------------------------------------------------------
  def speed_rate
    return @speed_rate if @speed_rate != nil
    re = LNX09::RE_ANISPEED =~ name
    @speed_rate = re ? [$1.to_i, 1].max : false
  end
end

#==============================================================================
# ■ Sprite_Base
#------------------------------------------------------------------------------
# 　アニメーションの表示処理を追加したスプライトのクラスです。
#==============================================================================

class Sprite_Base < Sprite
  #--------------------------------------------------------------------------
  # ● [再定義]:アニメーションの速度を設定
  #--------------------------------------------------------------------------
  def set_animation_rate
    default = LNX09::DEFAULT_MAP_SPEED_RATE
    @ani_rate = @animation.speed_rate ? @animation.speed_rate : default
  end
end
#==============================================================================
# ■ Sprite_Battler
#------------------------------------------------------------------------------
# 　バトラー表示用のスプライトです。
#==============================================================================

class Sprite_Battler < Sprite_Base
  #--------------------------------------------------------------------------
  # ● [オーバーライド]:アニメーションの速度を設定
  #--------------------------------------------------------------------------
  def set_animation_rate
    default = LNX09::DEFAULT_BATTLE_SPEED_RATE
    @ani_rate = @animation.speed_rate ? @animation.speed_rate : default
  end
end

#==============================================================================
# ★ RGSS3-Extension
# LNX10_バトルスピード調整
#==============================================================================

module LNX10
  #--------------------------------------------------------------------------
  # ● 設定
  #--------------------------------------------------------------------------
  # ウェイトの長さ(％)
  BASE_WAIT      = 80 # 規定値:80
  
  # 短時間ウェイトの長さ(フレーム)
  WAIT_SHORT     = 15 # 規定値:15
  
  # 早送りボタン(複数指定可) 
  FAST_KEY       = [:C, :A] # 規定値:[:C, :A] (=CボタンかAボタン)

  # 早送りの速さ
  FAST_SPEED     = 5.0 # 規定値:5.0
  
  # 早送り中、アニメーションやエフェクトも早送りする true = 有効 / false = 無効
  EFFECT_FAST    = true # 規定値：true

  # アニメーションとエフェクト表示の待ち時間省略(フレーム)
  ANIMATION_OMIT = 8  # 規定値:8
  EFFECT_OMIT    = 4  # 規定値:4
  
  # 早送り無効のウェイトを早送り可能にする true = 有効 / false = 無効
  ADS_FAST_FAST  = true
end

#==============================================================================
# ■ Game_Temp
#------------------------------------------------------------------------------
# 　セーブデータに含まれない、一時的なデータを扱うクラスです。このクラスのイン
# スタンスは $game_temp で参照されます。
#==============================================================================

class Game_Temp
  #--------------------------------------------------------------------------
  # ● [追加]:公開インスタンス変数
  #--------------------------------------------------------------------------
  attr_accessor :battle_fastforward  # 早送りしているか
  #--------------------------------------------------------------------------
  # ● [エイリアス]:オブジェクト初期化
  #--------------------------------------------------------------------------
  alias :lnx10_initialize :initialize
  def initialize
    lnx10_initialize
    @battle_fastforward = false
  end
end

#==============================================================================
# ■ Sprite_Battler
#------------------------------------------------------------------------------
# 　バトラー表示用のスプライトです。
#==============================================================================

class Sprite_Battler < Sprite_Base
  #--------------------------------------------------------------------------
  # ● [追加]:公開インスタンス変数
  #--------------------------------------------------------------------------
  attr_reader :ani_duration     # アニメーションの残り時間
  attr_reader :effect_duration  # エフェクトの残り時間
  #--------------------------------------------------------------------------
  # ● [オーバーライド]:アニメーションの更新
  #--------------------------------------------------------------------------
  def update_animation
    # スーパークラスのメソッドを呼ぶ
    super
    return unless $game_temp.battle_fastforward
    return unless animation?
    # 早送り中ならさらに1フレーム経過させる
    if @ani_duration % @ani_rate > 1 && LNX10::EFFECT_FAST
      @ani_duration -= @ani_duration % @ani_rate - 1
    end
  end
  #--------------------------------------------------------------------------
  # ● [エイリアス]:エフェクトの更新
  #--------------------------------------------------------------------------
  alias :lnx10_update_effect :update_effect
  def update_effect
    # 元のメソッドを呼ぶ
    lnx10_update_effect
    return unless $game_temp.battle_fastforward
    # 早送り中ならさらに1フレーム経過させる
    @effect_duration -= 1 if @effect_duration > 1 && LNX10::EFFECT_FAST
  end 
  #--------------------------------------------------------------------------
  # ● [再定義]:点滅エフェクトの更新
  #--------------------------------------------------------------------------
  def update_blink
    # アニメーション表示中に opacity が変化すると都合が悪いので、
    # flash で点滅させる
    self.opacity = 255
    self.flash(nil, 1) if @effect_duration % 10 >= 5
  end
end

#==============================================================================
# ■ Spriteset_Battle
#------------------------------------------------------------------------------
# 　バトル画面のスプライトをまとめたクラスです。
#==============================================================================

class Spriteset_Battle
  #--------------------------------------------------------------------------
  # ● [再定義]:アニメーション表示中判定
  #--------------------------------------------------------------------------
  def animation?
    # 残り時間が skip 以下の場合、再生中とみなさない
    skip = LNX10::ANIMATION_OMIT
    battler_sprites.any? {|sprite| sprite.ani_duration > skip}
  end
  #--------------------------------------------------------------------------
  # ● [再定義]:エフェクト実行中判定
  #--------------------------------------------------------------------------
  def effect?
    # 残り時間が skip 以下の場合、再生中とみなさない
    skip = LNX10::EFFECT_OMIT 
    battler_sprites.any? {|sprite| sprite.effect_duration > skip}
  end
end

#==============================================================================
# ■ Scene_Battle
#------------------------------------------------------------------------------
# 　バトル画面の処理を行うクラスです。
#==============================================================================

class Scene_Battle < Scene_Base
  #--------------------------------------------------------------------------
  # ● [追加]:ウェイトの長さを修正して返す
  #--------------------------------------------------------------------------
  def base_wait(duration)
    duration * LNX10::BASE_WAIT / 100
  end
  #--------------------------------------------------------------------------
  # ● [エイリアス]:フレーム更新（ウェイト用）
  #--------------------------------------------------------------------------
  alias :lnx10_update_for_wait :update_for_wait
  def update_for_wait
    # 元のメソッドを呼ぶ
    lnx10_update_for_wait
    # 早送り状態の更新
    show_fast?
  end
  #--------------------------------------------------------------------------
  # ● [再定義]:ウェイト
  #--------------------------------------------------------------------------
  def wait(duration)
    duration = base_wait(duration)
    skip = duration / LNX10::FAST_SPEED
    duration.times {|i| update_for_wait if i < skip || !show_fast? }
  end
  #--------------------------------------------------------------------------
  # ● [再定義]:早送り判定
  #--------------------------------------------------------------------------
  def show_fast?
    # 早送りボタンのどれか一つでも押されているか？
    fast = LNX10::FAST_KEY.collect {|key| Input.press?(key) }.include?(true)
    # Scene_Battle 以外のクラスで早送り情報を扱うために $game_temp に書き込む
    $game_temp.battle_fastforward = fast
  end
  #--------------------------------------------------------------------------
  # ● [再定義]:ウェイト（早送り無効）
  #--------------------------------------------------------------------------
  def abs_wait(duration)
    duration = base_wait(duration)
    if LNX10::ADS_FAST_FAST
      # 早送り無効ウェイトが早送り可能なら
      skip = duration / LNX10::FAST_SPEED
      duration.times {|i| update_for_wait if i < skip || !show_fast? }
    else
      # 早送り無効ウェイト
      duration.times {|i| update_for_wait }
    end
  end
  #--------------------------------------------------------------------------
  # ● [再定義]:短時間ウェイト（早送り無効）
  #--------------------------------------------------------------------------
  def abs_wait_short
    #wait_for_animation # アニメーションが再生されていたら待つ
    abs_wait(LNX10::WAIT_SHORT)
  end
end

#==============================================================================
# ■ [追加]:Popup_Data
#------------------------------------------------------------------------------
# 　戦闘中のポップアップをまとめて扱うクラス。ポップアップスプライトの
# initialize 時に自身を参照させて、ポップアップ内容を定義するのにも使います。
#==============================================================================

class Popup_Data 
  #--------------------------------------------------------------------------
  # ● 公開インスタンス変数
  #--------------------------------------------------------------------------
  attr_accessor :type       # ポップアップのタイプ
  attr_accessor :popup      # 表示する内容
  attr_accessor :popup_size # ポップアップの大きさ(シンボル指定)
  attr_accessor :color      # 色(シンボル指定)
  attr_accessor :deco       # 装飾文字
  attr_accessor :battler    # ポップアップするバトラー
  attr_accessor :delay      # 表示開始までの時間
  attr_accessor :viewport   # ビューポート
  #--------------------------------------------------------------------------
  # ● オブジェクト初期化
  #--------------------------------------------------------------------------
  def initialize
    @sprites = []
    spb = Sprite_PopupBase.new
    spb.create_number
    spb.dispose
    refresh
  end
  #--------------------------------------------------------------------------
  # ● スプライト解放
  #--------------------------------------------------------------------------
  def dispose
    @sprites.each {|sprite| sprite.dispose}
  end
  #--------------------------------------------------------------------------
  # ● フレーム更新
  #--------------------------------------------------------------------------
  def update
    @sprites.each do |sprite|
      sprite.update
      @sprites.delete(sprite) if sprite.disposed?
    end
  end
  #--------------------------------------------------------------------------
  # ● リフレッシュ
  #--------------------------------------------------------------------------
  def refresh
    @type  = 0
    @popup = nil
    @popup_size = :large
    @color = :hp_damage
    @deco = ["", -1] # [テキスト, テキストの位置]
    @battler = nil
    @delay = 0
    @viewport = nil    
  end
  #--------------------------------------------------------------------------
  # ● ポップアップを生成
  #--------------------------------------------------------------------------
  def make_popup
    case @type
    when 0
      # 通常のポップアップ
      @popup_size = :large
      @sprites.push(Sprite_PopupSpring.new(self))
    when 1
      # 小さいポップアップ
      @popup_size = :small
      @delay = @battler.popup_delay[1]
      @sprites.push(Sprite_PopupSpring.new(self))
    when 2
      # ゆっくり上昇(大)
      @popup_size = :large
      @delay = @battler.popup_delay[2]
      @sprites.push(Sprite_PopupRising.new(self))
    when 3
      # ゆっくり上昇(小)
      @popup_size = :small      
      @delay = @battler.popup_delay[2]
      @sprites.push(Sprite_PopupRising.new(self))
    when 4
      # スライド(大)
      @popup_size = :large
      @delay = @battler.popup_delay[3]
      @sprites.push(Sprite_PopupSliding.new(self))
    when 5
      # スライド(小)
      @popup_size = :small
      @delay = @battler.popup_delay[3]
      @sprites.push(Sprite_PopupSliding.new(self))      
    when 6
      # オーバーレイ(大)
      @popup_size = :large
      @sprites.push(Sprite_PopupOverlay.new(self))
    when 7
      # オーバーレイ(小)
      @popup_size = :small
      @sprites.push(Sprite_PopupOverlay.new(self))
    when 100
      # レベルアップ
      @battler.popup_delay[3] = 0
      @popup_size = :large
      @sprites.push(Sprite_PopupLevelUp.new(self))
    end
  end
  #--------------------------------------------------------------------------
  # ● TP のポップアップが有効か？
  #--------------------------------------------------------------------------
  def tp_popup_enabled?(target)
    return true if LNX11::TP_POPUP_TYPE == 0 # すべてポップアップ
    return true if LNX11::TP_POPUP_TYPE == 1 && target.actor? # アクターのみ
    false # ポップアップしない
  end
  #--------------------------------------------------------------------------
  # ● レベルアップのポップアップ
  #--------------------------------------------------------------------------
  def popup_levelup(target)
    refresh
    @type = 100
    @battler = target
    @popup = LNX11::POPUP_VOCAB[:levelup]
    @color = :levelup
    # ポップアップ作成
    make_popup
  end
  #--------------------------------------------------------------------------
  # ● 単一テキストのポップアップ
  #--------------------------------------------------------------------------
  def popup_text(target, type)
    refresh
    @type = LNX11::POPUP_TYPE[type]
    @battler = target
    @popup = LNX11::POPUP_VOCAB[type]
    @color = type
    # ポップアップ作成
    make_popup
  end
  #--------------------------------------------------------------------------
  # ● ミスのポップアップ
  #--------------------------------------------------------------------------
  def popup_miss(target, item)
    refresh
    @type = LNX11::POPUP_TYPE[:miss]
    @battler = target
    @popup = LNX11::POPUP_VOCAB[:miss]
    @color = :hp_damage
    # ポップアップ作成
    make_popup
  end
  #--------------------------------------------------------------------------
  # ● HP ダメージポップアップ
  #--------------------------------------------------------------------------
  def popup_hp_damage(target, item)
    return if target.result.hp_damage == 0 && item && !item.damage.to_hp?
    refresh
    @popup = target.result.hp_damage
    @battler = target
    if target.result.hp_drain > 0
      # 被吸収
      @type = LNX11::POPUP_TYPE[:hp_drain]
      # 弱点/耐性
      if target.result.element_rate > 1
        @deco = LNX11::DECORATION_NUMBER[:weakness]
        @color = :weakness
      elsif target.result.element_rate < 1
        @deco = LNX11::DECORATION_NUMBER[:resist]
        @color = :resist
      else
        @color = :hp_damage
      end
    elsif target.result.hp_damage > 0
      # ダメージ
      @type = LNX11::POPUP_TYPE[:hp_damage]
      @color = :hp_damage
      if target.result.critical
        # クリティカル
        @deco = LNX11::DECORATION_NUMBER[:critical]
        @color = :critical
      end
      # 弱点/耐性
      if target.result.element_rate > 1
        @deco = LNX11::DECORATION_NUMBER[:weakness]
        @color = :weakness if @color != :critical
      elsif target.result.element_rate < 1
        @deco = LNX11::DECORATION_NUMBER[:resist]
        @color = :resist if @color != :critical
      end
    elsif target.result.hp_damage < 0
      # 回復
      @type = LNX11::POPUP_TYPE[:hp_recovery]
      @color = :hp_recovery
    else
      # 0 ダメージ
      @type = LNX11::POPUP_TYPE[:hp_damage]
      @color = :hp_damage
    end 
    # ポップアップ作成
    make_popup
  end
  #--------------------------------------------------------------------------
  # ● MP ダメージポップアップ
  #--------------------------------------------------------------------------
  def popup_mp_damage(target, item)
    return if target.dead? || target.result.mp_damage == 0
    refresh
    @popup = target.result.mp_damage
    @battler = target
    @deco = LNX11::DECORATION_NUMBER[:mp_damage]
    if target.result.mp_drain > 0
      # 被吸収
      @type = LNX11::POPUP_TYPE[:mp_drain]
      @color = :mp_damage
    elsif target.result.mp_damage > 0
      # ダメージ
      @type = LNX11::POPUP_TYPE[:mp_damage]
      @color = :mp_damage
    elsif target.result.mp_damage < 0
      # 回復
      @type = LNX11::POPUP_TYPE[:mp_recovery]
      @color = :mp_recovery
    end
    # ポップアップ作成
    make_popup
  end
  #--------------------------------------------------------------------------
  # ● TP ダメージポップアップ
  #--------------------------------------------------------------------------
  def popup_tp_damage(target, item)
    return unless tp_popup_enabled?(target)
    return if target.dead? || target.result.tp_damage == 0
    refresh
    @popup = target.result.tp_damage
    @battler = target
    deco = target.result.tp_damage > 0 ? :tp_minus : :tp_plus
    @deco = LNX11::DECORATION_NUMBER[deco]
    @type = LNX11::POPUP_TYPE[:tp_damage]
    @color = :tp_damage
    # ポップアップ作成
    make_popup
  end
  #--------------------------------------------------------------------------
  # ● HP 吸収回復
  #--------------------------------------------------------------------------
  def popup_hp_drain(target, hp_drain)
    return if hp_drain == 0
    refresh
    @popup = hp_drain
    @battler = target
    @type = LNX11::POPUP_TYPE[:hp_drainrecv]
    @color = :hp_recovery
    # ポップアップ作成
    make_popup
  end
  #--------------------------------------------------------------------------
  # ● MP 吸収回復
  #--------------------------------------------------------------------------
  def popup_mp_drain(target, mp_drain)
    return if mp_drain == 0
    refresh
    @popup = mp_drain
    @battler = target
    @deco = LNX11::DECORATION_NUMBER[:mp_damage]
    @type = LNX11::POPUP_TYPE[:mp_drainrecv]
    @color = :mp_recovery
    # ポップアップ作成
    make_popup
  end
  #--------------------------------------------------------------------------
  # ● ステート付加のポップアップ
  #--------------------------------------------------------------------------
  def popup_added_states(target)
    refresh
    @battler = target
    target.result.added_state_objects.each do |state|
      next if state.id == target.death_state_id
      next if state.icon_index == 0 && LNX11::INDEXZERO_NO_POPUP
      next if state.add_no_display?      
      if state.add_no_decoration?
        @popup = state.display_name
      else
        @popup = sprintf(LNX11::DECORATION_TEXT[:add_state], state.display_name)
      end
      type = state.advantage? ? :add_state : :add_badstate
      if state.add_popup_type
        @type = state.add_popup_type
      else
        @type = LNX11::POPUP_TYPE[type]
      end
      @color = type
      # ポップアップ作成
      make_popup
    end
  end
  #--------------------------------------------------------------------------
  # ● ステート解除のポップアップ
  #--------------------------------------------------------------------------
  def popup_removed_states(target)
    refresh
    @type = LNX11::POPUP_TYPE[:rem_badstate]
    @battler = target
    target.result.removed_state_objects.each do |state|
      next if state.id == target.death_state_id
      next if state.icon_index == 0 && LNX11::INDEXZERO_NO_POPUP
      next if state.remove_no_display?
      if state.remove_no_decoration?
        @popup = state.display_name
      else
        @popup = sprintf(LNX11::DECORATION_TEXT[:rem_state], state.display_name)
      end
      type = state.advantage? ? :rem_state : :rem_badstate
      if state.remove_popup_type
        @type = state.remove_popup_type
      else
        @type = LNX11::POPUP_TYPE[type]
      end
      @color = type
      # ポップアップ作成
      make_popup
    end
  end
  #--------------------------------------------------------------------------
  # ● 能力強化／弱体のポップアップ
  #--------------------------------------------------------------------------
  def popup_buffs(target, buffs, fmt)
    return if buffs.empty?
    refresh
    @battler = target
    case fmt 
    when Vocab::BuffAdd 
      buffdeco = LNX11::DECORATION_TEXT[:add_buff]
      @type = LNX11::POPUP_TYPE[:add_buff]
      @color = :add_buff
    when Vocab::DebuffAdd
      buffdeco = LNX11::DECORATION_TEXT[:add_debuff]
      @type = LNX11::POPUP_TYPE[:add_debuff]
      @color = :add_debuff
    when Vocab::BuffRemove
      buffdeco = LNX11::DECORATION_TEXT[:rem_buff]
      @type = LNX11::POPUP_TYPE[:rem_buff]
      @color = :rem_buff
    end
    buffs.each do |param_id|
      @popup = sprintf(buffdeco, LNX11::POPUP_VOCAB_PARAMS[param_id])
      # ポップアップ作成
      make_popup
    end
  end
  #--------------------------------------------------------------------------
  # ● HP 再生
  #--------------------------------------------------------------------------
  def popup_regenerate_hp(target, hp_damage, paycost = false)
    return if hp_damage == 0
    refresh
    @popup = hp_damage
    @battler = target
    if hp_damage > 0
      # ダメージ
      @type = LNX11::POPUP_TYPE[:hp_slipdamage]
      @color = :hp_damage
    elsif hp_damage < 0
      # 回復
      @type = LNX11::POPUP_TYPE[:hp_regenerate]
      @color = :hp_recovery
    end 
    # ポップアップ作成
    make_popup
  end
  #--------------------------------------------------------------------------
  # ● MP 再生
  #--------------------------------------------------------------------------
  def popup_regenerate_mp(target, mp_damage, paycost = false)
    return if mp_damage == 0
    refresh
    @popup = mp_damage
    @battler = target
    @deco = LNX11::DECORATION_NUMBER[mp_damage > 0 ? :mp_minus : :mp_plus]
    if mp_damage > 0
      # ダメージ
      @type = LNX11::POPUP_TYPE[paycost ? :mp_paycost : :mp_slipdamage]
      @color = :mp_damage
    elsif mp_damage < 0
      # 回復
      @type = LNX11::POPUP_TYPE[paycost ? :mp_paycost : :mp_regenerate]
      @color = :mp_recovery
    end
    # ポップアップ作成
    make_popup
  end
  #--------------------------------------------------------------------------
  # ● TP 再生
  #--------------------------------------------------------------------------
  def popup_regenerate_tp(target, tp_damage, paycost = false)
    return unless tp_popup_enabled?(target)
    return if tp_damage == 0
    refresh
    @popup = tp_damage
    @battler = target
    @deco = LNX11::DECORATION_NUMBER[tp_damage > 0 ? :tp_minus : :tp_plus]
    @type = LNX11::POPUP_TYPE[paycost ? :tp_paycost : :tp_regenerate]
    @color = :tp_damage
    # ポップアップ作成
    make_popup
  end
  #--------------------------------------------------------------------------
  # ● TP チャージ
  #--------------------------------------------------------------------------
  def popup_tp_charge(target, tp_damage)
    return unless tp_popup_enabled?(target)
    return if tp_damage == 0
    refresh
    @popup = tp_damage
    @battler = target
    @deco = LNX11::DECORATION_NUMBER[tp_damage > 0 ? :tp_minus : :tp_plus]
    @type = LNX11::POPUP_TYPE[:tp_charge]
    @color = :tp_damage
    # ポップアップ作成
    make_popup
  end
  #--------------------------------------------------------------------------
  # ● TP 得
  #--------------------------------------------------------------------------
  def popup_tp_gain(target, tp_damage)
    return unless tp_popup_enabled?(target)
    return if tp_damage == 0
    refresh
    @popup = tp_damage
    @battler = target
    @deco = LNX11::DECORATION_NUMBER[tp_damage > 0 ? :tp_minus : :tp_plus]
    @type = LNX11::POPUP_TYPE[:tp_gain]
    @color = :tp_damage
    # ポップアップ作成
    make_popup
  end
end
#==============================================================================
# ■ [追加]:Sprite_PopupBase
#------------------------------------------------------------------------------
# 　戦闘中のダメージ表示等をポップアップ表示するためのスプライトの
# スーパークラス。サブクラスで細かい動きを定義します。
#==============================================================================

class Sprite_PopupBase < Sprite
  #--------------------------------------------------------------------------
  # ● クラス変数
  #--------------------------------------------------------------------------
  @@cache_number = []
  @@cache_text   = {}
  #--------------------------------------------------------------------------
  # ● 定数
  #--------------------------------------------------------------------------
  NUMBER_COLOR_SIZE = 8
  NUMBERS    = [0,1,2,3,4,5,6,7,8,9]
  COLOR_KEYS = LNX11::POPUP_COLOR.keys 
  #--------------------------------------------------------------------------
  # ● オブジェクト初期化
  #     data : ポップアップデータ
  #--------------------------------------------------------------------------
  def initialize(data = nil)
    return if data == nil
    # ポップアップデータを適用
    super(data.viewport)
    @battler = data.battler # ポップアップを表示するバトラー
    @delay   = data.delay   # ディレイ(遅延) ※サブクラスによって扱いが違います
    # 基本設定
    @duration = 60  # 消え始める時間
    @fadeout = 16   # 消える速さ
    @rx = ry = 0    # XY座標
    @rz = 350       # Z座標
    self.visible = false
    # ポップアップデータからビットマップを作成
    if data.popup.is_a?(Numeric)
      # ダメージ値
      self.bitmap = number(data.popup, data.color, data.popup_size, data.deco)
    elsif data.popup.is_a?(String)
      # テキスト
      self.bitmap = text(data.popup, data.color, data.popup_size)
    end
    # 位置設定
    set_position
  end
  #--------------------------------------------------------------------------
  # ● 解放
  #--------------------------------------------------------------------------
  def dispose
    self.bitmap.dispose if self.bitmap
    super
  end
  #--------------------------------------------------------------------------
  # ● フレーム更新
  #--------------------------------------------------------------------------
  def update
    update_xy
    @duration -= 1
    self.opacity -= @fadeout if @duration <= 0
    dispose if self.opacity == 0
  end
  #--------------------------------------------------------------------------
  # ● ディレイが残っている？
  # 　サブクラスの update メソッドで使用します。
  #--------------------------------------------------------------------------
  def delay?
    @delay -= 1
    self.visible = (@delay <= 0)
    !self.visible
  end
  #--------------------------------------------------------------------------
  # ● 座標更新
  #--------------------------------------------------------------------------
  def update_xy
    self.x = @rx
    self.y = @ry
    self.z = @rz
  end
  #--------------------------------------------------------------------------
  # ● ポップアップ位置の設定
  #--------------------------------------------------------------------------
  def set_position
    @rx = @battler.screen_x
    if @battler.actor?
      pos = LNX11::ACTOR_POPUP_POSITION
    else
      pos = [LNX11::ENEMY_POPUP_POSITION, 2].min
    end
    case pos
    when 0 ; @ry = @battler.screen_y        # 足元
    when 1 ; @ry = @battler.screen_y_center # 中心
    when 2 ; @ry = @battler.screen_y_top    # 頭上
    when 3 ; @ry = Graphics.height + LNX11::ACTOR_POPUP_Y # Y座標統一(アクター)
    end
  end
  #--------------------------------------------------------------------------
  # ● 数字ビットマップの取得
  #--------------------------------------------------------------------------
  def bitmap_number(size = :large)
    return @@cache_number[size == :large ? 0 : 1]
  end
  #--------------------------------------------------------------------------
  # ● 数字ビットマップの作成
  #--------------------------------------------------------------------------
  def create_number
    return if @@cache_number[0] && !@@cache_number[0].disposed?
    n_index = NUMBER_COLOR_SIZE
    @@cache_number.clear
    colors = LNX11::POPUP_COLOR.values # 色
    # 大・小の 2 パターンを作成する(ループ)
    [LNX11::LARGE_NUMBER, LNX11::SMALL_NUMBER].each do |n_size|
      w = n_size[:width]  # 数字の幅
      h = n_size[:height] # 数字の高さ
      # ビットマップ作成
      bitmap = Bitmap.new(w * NUMBERS.size, h * [colors.size, n_index].min)
      bitmap.font.name = LNX11::NUMBER_FONT
      bitmap.font.size = n_size[:fontsize]
      # 色ごとに分けて描画する(ループ)
      colors.size.times do |col|
        break if col >= n_index
        # 色を変更
        bitmap.font.color.set(colors[col][0])
        bitmap.font.out_color.set(colors[col][1])
        # 文字ごとに分けて描画(ループ)
        NUMBERS.size.times do |num|
          bitmap.draw_text(num * w, col * h, w, h, NUMBERS[num], 1)
        end
      end
      @@cache_number.push(bitmap)
    end
    p "LNX11a:数字ビットマップのキャッシュを作成しました。"
  end
  #--------------------------------------------------------------------------
  # ● ダメージ数を描画したビットマップの取得
  #--------------------------------------------------------------------------
  def number(num, color, size = :large, deco = nil)
    color_index = COLOR_KEYS.index(color)
    # ポップアップサイズを設定    
    n_bitmap = bitmap_number(size)
    n_size = size == :large ? LNX11::LARGE_NUMBER : LNX11::SMALL_NUMBER
    spacing = n_size[:spacing]
    w = n_size[:width]  # 数字の幅
    h = n_size[:height] # 数字の高さ
    # 数値を文字列の配列にする
    numbers = (num.abs.to_s).split(//)
    # 修飾文字があれば、テキストのビットマップを取得
    offset_x = offset_y = 0
    # ダメージ値の幅
    nw = w * numbers.size + spacing * (numbers.size - 1)
    case deco[1]
    when 0 # ダメージ値の上
      # テキストのビットマップを取得
      sizerate = LNX11::TEXT_SIZERATE[:pos_top]
      text_bitmap = text(deco[0], color, size = n_size[:fontsize] * sizerate)
      offset_y = text_bitmap.height * 0.6
      # 最終的なビットマップのサイズ
      bw = [nw, text_bitmap.width].max
      offset_x = bw / 2 - nw / 2
      bh = h + offset_y
    when 1 # ダメージ値の左
      # テキストのビットマップを取得
      sizerate = LNX11::TEXT_SIZERATE[:pos_left]
      text_bitmap = text(deco[0], color, size = n_size[:fontsize] * sizerate)
      offset_x = text_bitmap.width - 8
      # 最終的なビットマップのサイズ
      bw = nw + offset_x
      bh = h
    else # 修飾文字が無い
      # 最終的なビットマップのサイズ
      bw = nw
      bh = h
    end
    # ビットマップを作成
    bitmap = Bitmap.new(bw, bh)
    # 修飾文字を合成
    case deco[1]
    when 0 # ダメージ値の上
      bitmap.blt((bw - text_bitmap.width) / 2, 0, text_bitmap, text_bitmap.rect)
      text_bitmap.dispose
    when 1 # ダメージ値の左
      x = (text_bitmap.height - bh) * (bh > text_bitmap.height ? -1 : 1)
      bitmap.blt(0, x, text_bitmap, text_bitmap.rect)
      text_bitmap.dispose
    end
    # ダメージ値を描画
    rect = Rect.new(0, h * color_index, w, h)
    numbers.size.times do |n|
      rect.x = numbers[n].to_i * w
      bitmap.blt(w * n + spacing * n + offset_x, offset_y, n_bitmap, rect) 
    end
    bitmap
  end
  #--------------------------------------------------------------------------
  # ● テキストを描画したビットマップの取得
  # 　ステートやダメージ値の修飾文字の描画に使用します。
  #--------------------------------------------------------------------------
  def text(text, color, size = :large)
    # キャッシュがあればそれを返す(無ければ作成)
    key = text + color.to_s + size.to_s
    if @@cache_text[key] && !@@cache_text[key].disposed?
      return @@cache_text[key].clone 
    end
    text.gsub!("[HP]") { Vocab::hp_a } if text.include?("[HP]")
    text.gsub!("[MP]") { Vocab::mp_a } if text.include?("[MP]") 
    text.gsub!("[TP]") { Vocab::tp_a } if text.include?("[TP]")
    # テキストにマルチバイト文字があれば日本語用フォントを使う
    if text.length != text.bytesize
      fontname = LNX11::TEXT_FONT_J
    else
      fontname = LNX11::TEXT_FONT
    end
    # ポップアップサイズを設定
    sizerate = LNX11::TEXT_SIZERATE[:normal]
    case size
    when :large ; fontsize = LNX11::LARGE_NUMBER[:fontsize] * sizerate
    when :small ; fontsize = LNX11::SMALL_NUMBER[:fontsize] * sizerate
    else        ; fontsize = size
    end
    # テキストサイズ計算
    self.bitmap = Cache.empty_bitmap
    self.bitmap.font.name = fontname
    self.bitmap.font.size = fontsize
    rect = self.bitmap.text_size(text)
    rect.width += 8
    self.bitmap.dispose
    # ビットマップを作成
    bitmap = Bitmap.new(rect.width, rect.height) 
    bitmap.font.name = fontname
    bitmap.font.size = fontsize
    bitmap.font.color.set(LNX11::POPUP_COLOR[color][0])
    bitmap.font.out_color.set(LNX11::POPUP_COLOR[color][1])
    # テキスト描画
    bitmap.draw_text(rect, text, 1)
    # キャッシュに保存
    @@cache_text[key] = bitmap
    # ビットマップを返す
    bitmap.clone
  end
end

#==============================================================================
# ■ [追加]:Sprite_PopupSpring
#------------------------------------------------------------------------------
# 　通常の跳ねるポップアップ。
#==============================================================================

class Sprite_PopupSpring < Sprite_PopupBase
  #--------------------------------------------------------------------------
  # ● オブジェクト初期化
  #--------------------------------------------------------------------------
  def initialize(data)
    super(data)
    # 動き設定
    set_movement(data.popup_size)
    @rz += data.popup_size == :large ? 8 : 7
    self.ox = self.width  / 2
    self.oy = self.height / 2 - @battler.popup_delay[1]
    # ディレイ設定 同じタイプのポップアップが重ならないようにする
    if data.popup_size == :small
      @delay_clear = (@delay == 0)
      dy = self.bitmap.height * 0.8
      @battler.popup_delay[1] += dy
      @delay = 0
    end
  end
  #--------------------------------------------------------------------------
  # ● 投射運動の設定
  #--------------------------------------------------------------------------
  def set_movement(size = :large)
    movement = size == :large ? LNX11::LARGE_MOVEMENT : LNX11::SMALL_MOVEMENT
    @fall       = -movement[:inirate]
    @gravity    =  movement[:gravity]
    @side       =  movement[:side_scatter] * rand(0) * (rand(2) == 0 ? -1 : 1)
    @ref_move   =  movement[:ref_height]
    @ref_factor =  movement[:ref_factor]
    @ref_count  =  movement[:ref_count]
    @duration   =  movement[:duration]
    @fadeout    =  movement[:fadeout]
  end
  #--------------------------------------------------------------------------
  # ● 解放
  #--------------------------------------------------------------------------  
  def dispose
    # 一番上のポップアップなら、ディレイを初期化する
    @battler.popup_delay[1] = 0 if @delay_clear
    super
  end
  #--------------------------------------------------------------------------
  # ● フレーム更新
  #--------------------------------------------------------------------------
  def update
    return if delay?
    update_freefall
    super
  end
  #--------------------------------------------------------------------------
  # ● 投射運動の更新
  #--------------------------------------------------------------------------
  def update_freefall
    if @ref_count >= 0
      # X:左右移動
      @rx += @side
      # Y:自由落下
      @ry += @fall
      @ref_move -= @fall
      @fall += @gravity
      # 跳ね返り
      if @ref_move <= 0 && @fall >= 0
        @ref_count -= 1
        @fall = -@fall * @ref_factor
      end      
    end
  end
end

#==============================================================================
# ■ [追加]:Sprite_PopupRising
#------------------------------------------------------------------------------
# 　少しずつ上昇するポップアップ。
#==============================================================================

class Sprite_PopupRising < Sprite_PopupBase
  #--------------------------------------------------------------------------
  # ● オブジェクト初期化
  #--------------------------------------------------------------------------
  def initialize(data)
    super(data)
    # 動き設定
    set_movement
    @rz += data.popup_size == :large ? 2 : 1
    self.ox = self.width  / 2
    self.oy = self.height / 2
    self.opacity = 0
    d = self.bitmap.height * LNX11::RISE_MOVEMENT[:line_spacing]
    @delay_count = d / @rising_speed
    @battler.popup_delay[2] += @delay_count
  end
  #--------------------------------------------------------------------------
  # ● 動きの設定
  #--------------------------------------------------------------------------
  def set_movement
    @rising_speed = LNX11::RISE_MOVEMENT[:rising_speed]
    @duration = LNX11::RISE_MOVEMENT[:duration]
    @fadeout  = LNX11::RISE_MOVEMENT[:fadeout]
  end
  #--------------------------------------------------------------------------
  # ● フレーム更新
  #--------------------------------------------------------------------------
  def update
    return if delay?
    @delay_count -= 1
    @battler.popup_delay[2] -= 1 if @delay_count >= 0
    @ry -= @rising_speed # すこしずつ上昇
    self.opacity += 32 if @duration > 0
    super
  end
end

#==============================================================================
# ■ [追加]:Sprite_PopupSliding
#------------------------------------------------------------------------------
# 　スライド表示するポップアップ。
#==============================================================================

class Sprite_PopupSliding < Sprite_PopupBase
  #--------------------------------------------------------------------------
  # ● オブジェクト初期化
  #--------------------------------------------------------------------------
  def initialize(data)
    super(data)
    # 動き設定
    set_movement
    @rz += data.popup_size == :large ? 6 : 5
    self.ox = self.width  / 2
    self.oy = self.height / 2 + (@delay)
    self.opacity = 0
    # ディレイ設定 同じタイプのポップアップが重ならないようにする
    @delay_clear = (@delay == 0)
    dy = self.bitmap.height * LNX11::SLIDE_MOVEMENT[:line_spacing]
    @battler.popup_delay[3] += dy
    @delay = 0
  end
  #--------------------------------------------------------------------------
  # ● 動きの設定
  #--------------------------------------------------------------------------
  def set_movement
    @x_speed  = LNX11::SLIDE_MOVEMENT[:x_speed]
    @duration = LNX11::SLIDE_MOVEMENT[:duration]
    @fadeout  = LNX11::SLIDE_MOVEMENT[:fadeout]
    # フェードインのスライド分だけ X を移動
    @rx -= @x_speed * 255.0 / @fadeout
  end
  #--------------------------------------------------------------------------
  # ● 解放
  #--------------------------------------------------------------------------  
  def dispose
    # 一番下のポップアップなら、ディレイを初期化する
    @battler.popup_delay[3] = 0 if @delay_clear
    super
  end
  #--------------------------------------------------------------------------
  # ● フレーム更新
  #--------------------------------------------------------------------------
  def update
    return if delay?
    @rx += @x_speed if opacity != 255 # スライド
    self.opacity += @fadeout if @duration > 0
    super
  end
end

#==============================================================================
# ■ [追加]:Sprite_PopupOverlay
#------------------------------------------------------------------------------
# 　オーバーレイポップアップ。自身の複製を加算合成で重ね合わせます。
#==============================================================================

class Sprite_PopupOverlay < Sprite_PopupBase
  #--------------------------------------------------------------------------
  # ● オブジェクト初期化
  #--------------------------------------------------------------------------
  def initialize(data)
    super(data)
    @duration = LNX11::OVERLAY_MOVEMENT[:duration]
    @fadeout  = LNX11::OVERLAY_MOVEMENT[:fadeout]
    @rz += data.popup_size == :large ? 4 : 3
    self.ox = self.width  / 2
    self.oy = self.height / 2
    # オーバーレイの作成
    create_overlay
  end
  #--------------------------------------------------------------------------
  # ● オーバーレイ(複製)の作成
  #--------------------------------------------------------------------------
  def create_overlay
    @overlay = Sprite.new
    @overlay.bitmap   = self.bitmap
    @overlay.viewport = self.viewport
    @overlay.ox = self.ox
    @overlay.oy = self.oy
    @overlay.visible = false
    @overlay.blend_type = 1
  end
  #--------------------------------------------------------------------------
  # ● 解放
  #--------------------------------------------------------------------------
  def dispose
    @overlay.dispose
    @overlay = nil
    super
  end
  #--------------------------------------------------------------------------
  # ● フレーム更新
  #--------------------------------------------------------------------------
  def update
    return if delay?
    self.opacity += @fadeout * 2 if @duration > 0
    super
    update_overlay
  end
  #--------------------------------------------------------------------------
  # ● オーバーレイの更新
  #--------------------------------------------------------------------------
  def update_overlay
    return if self.disposed? || @overlay.opacity == 0
    @overlay.x = self.x
    @overlay.y = self.y
    @overlay.z = self.z + 6
    # 拡大・消去
    @overlay.zoom_x += 0.16
    @overlay.zoom_y += 0.12
    @overlay.opacity -= 20
    @overlay.visible = (@overlay.opacity != 0)
  end
end

#==============================================================================
# ■ [追加]:Sprite_PopupLevelUp
#------------------------------------------------------------------------------
# 　レベルアップのポップアップ。スライドポップアップを継承しています。
#==============================================================================

class Sprite_PopupLevelUp < Sprite_PopupSliding
  #--------------------------------------------------------------------------
  # ● オブジェクト初期化
  #--------------------------------------------------------------------------
  def initialize(data)
    super(data)
  end
  #--------------------------------------------------------------------------
  # ● ポップアップ位置の設定
  #--------------------------------------------------------------------------
  def set_position
    @rx = @battler.screen_x - LNX11::ACTOR_OFFSET[:x]
    case LNX11::LEVELUP_POPUP_POSITION
    when 0 ; @ry = @battler.screen_y        # 足元
    when 1 ; @ry = @battler.screen_y_center # 中心
    when 2 ; @ry = @battler.screen_y_top    # 頭上
    when 3 ; @ry = Graphics.height + LNX11::LEVELUP_POPUP_Y # Y座標統一
    end
  end
  #--------------------------------------------------------------------------
  # ● フレーム更新
  #--------------------------------------------------------------------------
  def update
    super
    # リザルト中なら自動的に消去しない
    @duration += 1 if $game_troop.all_dead? || $game_message.busy?
  end
end

#==============================================================================
# ■ Bitmap
#==============================================================================

class Bitmap
  #--------------------------------------------------------------------------
  # ● [追加]:カーソル用三角形の描画
  #--------------------------------------------------------------------------
  def lnx_cursor_triangle(size, color, oy = 0, grad = 0)
    color = color.clone
    x = (self.width - size) / 2
    y = (self.height - size) / 2 + oy
    rect = Rect.new(x, y, size, 1)
    (size / 2).times do
      clear_rect(rect)
      fill_rect(rect, color)
      color.red   = [color.red   - 4.0 * grad, 0].max
      color.green = [color.green - 3.5 * grad, 0].max
      color.blue  = [color.blue  - 3.0 * grad, 0].max
      rect.y += rect.height
      clear_rect(rect)
      fill_rect(rect, color) 
      color.red   = [color.red   - 4.0 * grad, 0].max
      color.green = [color.green - 3.5 * grad, 0].max
      color.blue  = [color.blue  - 3.0 * grad, 0].max
      rect.x += 1
      rect.y += rect.height
      rect.width -= 2
    end
  end
end

#==============================================================================
# ■ [追加]:Sprite_TargetCursor
#------------------------------------------------------------------------------
# 　対象選択されているバトラーや行動選択中のアクターを示すアローカーソルです。
#==============================================================================
class Sprite_TargetCursor < Sprite
  @@cursor_cache = nil
  #--------------------------------------------------------------------------
  # ● オブジェクト初期化
  #--------------------------------------------------------------------------
  def initialize(viewport = nil)
    super(viewport)
    @wait = LNX11::CURSOR_ANI_SPEED
    @speed = [LNX11::CURSOR_SPEED, 1].max
    @battler = nil
    self.bitmap = cursor_bitmap
    partition = self.bitmap.width / self.height
    self.src_rect.set(0, 0, self.width / partition, self.height)
    self.ox = self.width / 2
    self.oy = self.height / 2
    self.x = @rx = @tx = 0
    self.y = @ry = @ty = 0
    self.z = 98
    self.visible = false
  end
  #--------------------------------------------------------------------------
  # ● ビットマップの設定
  #--------------------------------------------------------------------------
  def cursor_bitmap
    if !LNX11::CURSOR_NAME.empty?
      return Cache.system(LNX11::CURSOR_NAME)
    else
      # カーソルファイル名が指定されていなければRGSS側で生成
      return @@cursor_cache if @@cursor_cache && !@@cursor_cache.disposed?
      @@cursor_cache = Bitmap.new(32, 32)
      color = Color.new(0, 0, 0)
      @@cursor_cache.lnx_cursor_triangle(26, color, 2)
      2.times {@@cursor_cache.blur}
      color.set(255, 255, 255)
      @@cursor_cache.lnx_cursor_triangle(24, color, 1)
      color.set(104, 144, 180, 232)
      @@cursor_cache.lnx_cursor_triangle(20, color,  0, 2)
      @@cursor_cache.lnx_cursor_triangle(20, color,  1, 2)
      p "LNX11a:カーソルビットマップを作成しました。"
      return @@cursor_cache
    end
  end
  #--------------------------------------------------------------------------
  # ● カーソルの表示
  #--------------------------------------------------------------------------
  def show
    self.x = @rx = @tx
    self.y = @ry = @ty
    self.visible = true
  end
  #--------------------------------------------------------------------------
  # ● カーソルの非表示
  #--------------------------------------------------------------------------
  def hide
    @battler = nil
    self.visible = false
  end
  #--------------------------------------------------------------------------
  # ● 移動平均
  #--------------------------------------------------------------------------
  def sma(a, b, p)
    # a = 目標位置 b = 現在地
    return a if a == b || (a - b).abs < 0.3 || p == 1
    result = ((a + b * (p.to_f - 1)) / p.to_f)
    return (a - result).abs <= 1.0 ? (b < a ? b + 0.3 : b - 0.3) : result
  end
  #--------------------------------------------------------------------------
  # ● フレーム更新
  #--------------------------------------------------------------------------
  def update
    return unless visible
    super
    # アニメーションを進める
    @wait -= 1
    if @wait <= 0
      @wait += LNX11::CURSOR_ANI_SPEED
      self.src_rect.x += self.width
      self.src_rect.x = 0 if self.src_rect.x >= self.bitmap.width
    end
    # カーソルの座標を更新
    if @battler != nil
      @tx = @battler.screen_x
      @ty = [@battler.screen_y_top, 48 + self.oy].max
    end
    self.x = @rx = sma(@tx, @rx, @speed)
    self.y = @ry = sma(@ty, @ry, @speed)
  end
  #--------------------------------------------------------------------------
  # ● カーソル位置の設定
  #--------------------------------------------------------------------------
  def set(*args)
    if args.size >= 2
      # 引数が2つなら、XY指定
      @battler = nil
      @tx = args[0].screen_x
      @ty = args[1].screen_y
    elsif @battler != args[0]
      # 引数が1つなら、バトラー指定
      if args[0]
        @battler = args[0]
        @tx = @battler.screen_x
        @ty = @battler.screen_y_top
      else
        hide
      end
    end
    if @speed == 1 || !visible
      # スピードが1かカーソルが非表示なら表示に変える
      show
    end
  end
end

#==============================================================================
# ■ [追加]:Sprite_OneLine_BattleLog
#------------------------------------------------------------------------------
# 　バトルログを動的に表示するスプライトです。単体では1行しか表示できませんが、
# 複数同時に扱うことで複数行の表示を実装しています。
#==============================================================================

class Sprite_OneLine_BattleLog < Sprite
  @@grad_cache = nil
  #--------------------------------------------------------------------------
  # ● 公開インスタンス変数
  #--------------------------------------------------------------------------
  attr_accessor   :position # 表示位置(行)
  #--------------------------------------------------------------------------
  # ● オブジェクト初期化
  #--------------------------------------------------------------------------
  def initialize(width, height, max_line_number)
    super(nil)
    @rx = @ry = 0
    @line_height = height
    @max_line_number = max_line_number
    @position = -1
    @visible = true
    self.ox = -LNX11::STORAGE_OFFSET[:x]
    self.oy = -LNX11::STORAGE_OFFSET[:y]
    self.opacity = 0
    self.z = 96
    self.bitmap = Bitmap.new(width * 0.75, height)
  end
  #--------------------------------------------------------------------------
  # ● 解放
  #--------------------------------------------------------------------------  
  def dispose
    self.bitmap.dispose
    super
  end
  #--------------------------------------------------------------------------
  # ● スプライトの表示
  #--------------------------------------------------------------------------  
  def show
    @visible = true
  end
  #--------------------------------------------------------------------------
  # ● スプライトの非表示
  #--------------------------------------------------------------------------  
  def hide
    @visible = false
    @position = -1
  end
  #--------------------------------------------------------------------------
  # ● 表示しているか？
  #--------------------------------------------------------------------------  
  def visible?
    @visible
  end
  #--------------------------------------------------------------------------
  # ● フレーム更新
  #--------------------------------------------------------------------------
  def update
    return if self.opacity == 0
    self.opacity += @visible && @position >= 0 ? 24 : -24
    self.visible = self.opacity > 0
    return unless @visible # 不可視状態なら座標を更新しない
    @ry = (target_y + (@ry * 5)) / 6.0 if target_y < @ry
    @rx += 2 if @rx < 0
    self.x = @rx
    self.y = @ry
  end
  #--------------------------------------------------------------------------
  # ● テキストの描画
  # 　自身が持つ Bitmap で描画するのではなく、Window の contents から
  # コピーします(Sprite からでは Window_Base#draw_text_ex が扱えないため)。
  #--------------------------------------------------------------------------
  def set_text(window, position)
    self.bitmap.clear
    # 横グラデーション
    if @@grad_cache && !@@grad_cache.disposed?
      self.bitmap.blt(0, 0, @@grad_cache, self.bitmap.rect)
    else
      color = LNX11::STORAGE_GRADIENT_COLOR
      fillrect = self.bitmap.rect
      fillrect.width /= 2
      self.bitmap.gradient_fill_rect(fillrect, color[0], color[1])
      @@grad_cache = self.bitmap.clone
    end
    # contents からコピー
    self.bitmap.blt(4, 0, window.contents, self.bitmap.rect)
    self.opacity = 1
    @rx = -8
    @position = position
    @ry = target_y
  end
  #--------------------------------------------------------------------------
  # ● 位置の繰り上げ
  #--------------------------------------------------------------------------
  def up_position
    @position -= 1
  end
  #--------------------------------------------------------------------------
  # ● メッセージが空か？(表示待ち)
  #--------------------------------------------------------------------------
  def mes_empty?
    @position < 0
  end
  #--------------------------------------------------------------------------
  # ● 移動するべき Y 座標
  #--------------------------------------------------------------------------
  def target_y
    @position * @line_height
  end
end

#==============================================================================
# ■ Window_BattleLog
#------------------------------------------------------------------------------
# 　戦闘の進行を実況表示するウィンドウです。
#==============================================================================

class Window_BattleLog < Window_Selectable
  #--------------------------------------------------------------------------
  # ● [エイリアス]:オブジェクト初期化
  #--------------------------------------------------------------------------
  alias :lnx11a_initialize :initialize
  def initialize
    lnx11a_initialize
    # バトルログタイプが 1 以上なら非表示にする
    hide if LNX11::BATTLELOG_TYPE >= 1
  end
  #--------------------------------------------------------------------------
  # ● [追加]:文章の配列のクリア
  #--------------------------------------------------------------------------
  def lines_clear
    @lines.clear if LNX11::BATTLELOG_TYPE == 1
  end
  #--------------------------------------------------------------------------
  # ● [再定義]:メッセージ速度の取得
  #--------------------------------------------------------------------------
  def message_speed
    return LNX11::BATTLELOG_TYPE == 2 ? 15 : 20
  end
#==============================================================================
# バトルログタイプが 1:蓄積型 の場合は定義
#==============================================================================
if LNX11::BATTLELOG_TYPE == 1
  #--------------------------------------------------------------------------
  # ● [オーバーライド]:ウィンドウ高さの取得
  #--------------------------------------------------------------------------
  def window_height
    fitting_height(1)
  end
  #--------------------------------------------------------------------------
  # ● [オーバーライド]:最大行数の取得
  #--------------------------------------------------------------------------
  def max_line_number
    return LNX11::STORAGE_LINE_NUMBER
  end
  #--------------------------------------------------------------------------
  # ● [オーバーライド]:行の高さを取得
  #--------------------------------------------------------------------------
  def line_height
    return LNX11::STORAGE_LINE_HEIGHT
  end
  #--------------------------------------------------------------------------
  # ● [オーバーライド]:フォント設定のリセット
  #--------------------------------------------------------------------------
  def reset_font_settings
    super
    contents.font.size = LNX11::STORAGE_FONT[:size]
    contents.font.out_color.set(LNX11::STORAGE_FONT[:out_color])
  end
  #--------------------------------------------------------------------------
  # ● [再定義]:背景スプライトの作成
  #--------------------------------------------------------------------------
  def create_back_sprite
    # 背景スプライトは使用しない
    create_message_sprite
  end
  #--------------------------------------------------------------------------
  # ● [再定義]:背景スプライトの解放
  #--------------------------------------------------------------------------
  def dispose_back_sprite
    # 背景スプライトは使用しない
    dispose_message_sprite
  end
  #--------------------------------------------------------------------------
  # ● [追加]:メッセージスプライトの作成
  #--------------------------------------------------------------------------
  def create_message_sprite
    # メッセージスプライト 行数分だけ作成する
    @mes_position = 0 # 次にメッセージを表示させる位置
    @mesup_count = LNX11::STORAGE_UP_MESSAGE_TIME # ログが進行するまでの時間
    @mes_sprites = Array.new(max_line_number + 1) {
    Sprite_OneLine_BattleLog.new(self.width, line_height, max_line_number)}     
  end
  #--------------------------------------------------------------------------
  # ● [追加]:メッセージスプライトの解放
  #--------------------------------------------------------------------------
  def dispose_message_sprite
    @mes_sprites.each {|sprite| sprite.dispose }
  end
  #--------------------------------------------------------------------------
  # ● [追加]:メッセージスプライトを表示
  #--------------------------------------------------------------------------
  def show_message_sprite
    @mes_sprites.each {|sprite| sprite.show }
  end
  #--------------------------------------------------------------------------
  # ● [追加]:メッセージスプライトを非表示
  #--------------------------------------------------------------------------
  def hide_message_sprite
    @mes_sprites.each {|sprite| sprite.hide }
    @mes_position = 0
  end
  #--------------------------------------------------------------------------
  # ● [追加]:メッセージスプライトを一つ進める
  #--------------------------------------------------------------------------
  def up_message_sprite
    @mes_sprites.each {|sprite| sprite.up_position }
  end
  #--------------------------------------------------------------------------
  # ● [追加]:空のメッセージスプライトを返す
  #--------------------------------------------------------------------------
  def empty_message_sprite
    @mes_sprites.each {|sprite| return sprite if sprite.mes_empty? }
    @mes_sprites[0]
  end
  #--------------------------------------------------------------------------
  # ● [オーバーライド]:フレーム更新
  #--------------------------------------------------------------------------
  def update
    super
    # バトルログ消去フラグが有効
    if $game_temp.battlelog_clear
      $game_temp.battlelog_clear = false
      # スプライトが表示されていれば非表示にする
      if @mes_sprites[0].visible?
        hide_message_sprite
        lines_clear
      end
    end
    # ログの自動進行
    @mesup_count -= 1
    if @mesup_count <= 0 && @mes_position > 0
      up_message_sprite
      @mes_position -= 1
      @mesup_count = LNX11::STORAGE_UP_MESSAGE_TIME
    end
    @mes_sprites.each {|sprite| sprite.update }
  end
  #--------------------------------------------------------------------------
  # ● [再定義]:リフレッシュ
  #--------------------------------------------------------------------------
  def refresh
    # 文章が無ければ何もしない
    return if @lines.empty? 
    # スプライトを表示する
    show_message_sprite unless @mes_sprites[0].visible? 
    # 文章の描画
    contents.clear
    @lines[0] = last_text
    return if @lines[0].empty? 
    draw_line(0)
    # ウィンドウの内容をスプライトにコピー
    empty_message_sprite.set_text(self, @mes_position)
    # スプライト位置の変動    
    if @mes_position < max_line_number
      @mes_position += 1
    elsif @mesup_count > 0
      up_message_sprite
    end
    @mesup_count = (LNX11::STORAGE_UP_MESSAGE_TIME * 1.5).truncate
  end  
  #--------------------------------------------------------------------------
  # ● [再定義]:クリア
  #--------------------------------------------------------------------------
  def clear
    # なにもしない
  end
  #--------------------------------------------------------------------------
  # ● [再定義]:一行戻る
  #--------------------------------------------------------------------------
  def back_one
    # なにもしない
  end
  #--------------------------------------------------------------------------
  # ● [再定義]:指定した行に戻る
  #--------------------------------------------------------------------------
  def back_to(line_number)
    # なにもしない
  end
end
#==============================================================================
# バトルログタイプが 2:非表示 の場合は定義
#==============================================================================
if LNX11::BATTLELOG_TYPE == 2
  #--------------------------------------------------------------------------
  # ● [再定義]:リフレッシュ
  #--------------------------------------------------------------------------
  def refresh
    # なにもしない
  end  
  #--------------------------------------------------------------------------
  # ● [再定義]:クリア
  #--------------------------------------------------------------------------
  def clear
    # なにもしない
  end
end
#==============================================================================
  #--------------------------------------------------------------------------
  # ● [追加]:アニメーションのウェイト用メソッドの設定
  #--------------------------------------------------------------------------
  def method_wait_for_animation=(method)
    @method_wait_for_animation = method
  end
  #--------------------------------------------------------------------------
  # ● [追加]:アニメーション再生が終わるまでウェイト
  #--------------------------------------------------------------------------
  def wait_for_animation
    @method_wait_for_animation.call if @method_wait_for_animation
  end
  #--------------------------------------------------------------------------
  # ● [エイリアス]:スキル／アイテム使用の表示
  # 　使用時アニメーションの処理を追加します。
  #--------------------------------------------------------------------------
  alias :lnx11a_display_use_item :display_use_item
  def display_use_item(subject, item)
    if item.use_animation > 0
      # 使用時アニメーションが設定されていれば再生
      subject.animation_id = item.use_animation
      subject.animation_mirror = false
    end
    # 元のメソッドを呼ぶ
    lnx11a_display_use_item(subject, item)
    # アニメーションのウェイト
    wait_for_animation if item.use_animation > 0
  end
  #--------------------------------------------------------------------------
  # ● [追加]:ポップアップデータの取得
  #--------------------------------------------------------------------------
  def popup_data
    $game_temp.popup_data
  end
  #--------------------------------------------------------------------------
  # ● [エイリアス]:反撃の表示
  #--------------------------------------------------------------------------
  alias :lnx11a_display_counter :display_counter
  def display_counter(target, item)
    # ポップアップ
    popup_data.popup_text(target, :counter)
    # 元のメソッドを呼ぶ
    lnx11a_display_counter(target, item)
  end
  #--------------------------------------------------------------------------
  # ● [エイリアス]:反射の表示
  #--------------------------------------------------------------------------
  alias :lnx11a_display_reflection :display_reflection
  def display_reflection(target, item)
    # ポップアップ
    popup_data.popup_text(target, :reflection)
    # 元のメソッドを呼ぶ
    lnx11a_display_reflection(target, item)
  end
  #--------------------------------------------------------------------------
  # ● [エイリアス]:身代わりの表示
  #--------------------------------------------------------------------------
  alias :lnx11a_display_substitute :display_substitute
  def display_substitute(substitute, target)
    # ポップアップ
    popup_data.popup_text(substitute, :substitute)
    # 元のメソッドを呼ぶ
    lnx11a_display_substitute(substitute, target)
  end
  #--------------------------------------------------------------------------
  # ● [エイリアス]:失敗の表示
  #--------------------------------------------------------------------------
  alias :lnx11a_display_failure :display_failure
  def display_failure(target, item)
    if target.result.hit? && !target.result.success
      # ポップアップ (ミスと同じ扱いにする)
      popup_data.popup_miss(target, item)
    end
    # 元のメソッドを呼ぶ
    lnx11a_display_failure(target, item)
  end
  #--------------------------------------------------------------------------
  # ● [エイリアス]:ミスの表示
  #--------------------------------------------------------------------------
  alias :lnx11a_display_miss :display_miss
  def display_miss(target, item)
    # ポップアップ
    popup_data.popup_miss(target, item)
    # 元のメソッドを呼ぶ    
    lnx11a_display_miss(target, item)
  end
  #--------------------------------------------------------------------------
  # ● [エイリアス]:回避の表示
  #--------------------------------------------------------------------------
  alias :lnx11a_display_evasion :display_evasion
  def display_evasion(target, item)
    # ポップアップ (ミスと同じ扱いにする)
    popup_data.popup_miss(target, item)
    # 元のメソッドを呼ぶ    
    lnx11a_display_evasion(target, item)
  end
  #--------------------------------------------------------------------------
  # ● [エイリアス]:HP ダメージ表示
  #--------------------------------------------------------------------------
  alias :lnx11a_display_hp_damage :display_hp_damage
  def display_hp_damage(target, item)
    # ポップアップ
    popup_data.popup_hp_damage(target, item)
    # 元のメソッドを呼ぶ
    lnx11a_display_hp_damage(target, item)
  end
  #--------------------------------------------------------------------------
  # ● [エイリアス]:MP ダメージ表示
  #--------------------------------------------------------------------------
  alias :lnx11a_display_mp_damage :display_mp_damage
  def display_mp_damage(target, item)
    # ポップアップ
    popup_data.popup_mp_damage(target, item)
    # 元のメソッドを呼ぶ
    lnx11a_display_mp_damage(target, item)
  end
  #--------------------------------------------------------------------------
  # ● [エイリアス]:TP ダメージ表示
  #--------------------------------------------------------------------------
  alias :lnx11a_display_tp_damage :display_tp_damage
  def display_tp_damage(target, item)
    # ポップアップ
    popup_data.popup_tp_damage(target, item)
    # 元のメソッドを呼ぶ
    lnx11a_display_tp_damage(target, item)
  end
  #--------------------------------------------------------------------------
  # ● [エイリアス]:ステート付加の表示
  #--------------------------------------------------------------------------
  alias :lnx11a_display_added_states :display_added_states
  def display_added_states(target)
    # ポップアップ
    popup_data.popup_added_states(target)
    # 元のメソッドを呼ぶ
    lnx11a_display_added_states(target)
  end
  #--------------------------------------------------------------------------
  # ● [エイリアス]:ステート解除の表示
  #--------------------------------------------------------------------------
  alias :lnx11a_display_removed_states :display_removed_states
  def display_removed_states(target)
    # ポップアップ
    popup_data.popup_removed_states(target)
    # 元のメソッドを呼ぶ
    lnx11a_display_removed_states(target)
  end
  #--------------------------------------------------------------------------
  # ● [エイリアス]:能力強化／弱体の表示（個別）
  #--------------------------------------------------------------------------
  alias :lnx11a_display_buffs :display_buffs
  def display_buffs(target, buffs, fmt)
    # ポップアップ
    popup_data.popup_buffs(target, buffs, fmt)
    # 元のメソッドを呼ぶ
    lnx11a_display_buffs(target, buffs, fmt)
  end
end

#==============================================================================
# ■ RPG::BaseItem
#------------------------------------------------------------------------------
# 　アクター、職業、スキル、アイテム、武器、防具、敵キャラ、およびステートの
# スーパークラス。
#==============================================================================

class RPG::BaseItem  
  #--------------------------------------------------------------------------
  # ● [追加]:バトラーグラフィックファイル名を取得
  #--------------------------------------------------------------------------
  def default_battler_graphic
    # キャッシュがある場合、それを返す
    return @default_battler_graphic if @default_battler_graphic
    # メモ取得
    re = LNX11::RE_BATTLER =~ note
    @default_battler_graphic = re ? $1 : ""
  end
  #--------------------------------------------------------------------------
  # ● [追加]:使用時アニメの取得
  #--------------------------------------------------------------------------
  def use_animation
    # キャッシュがある場合、それを返す
    return @use_animation if @use_animation
    # メモ取得
    re = LNX11::RE_USE_ANIMATION =~ note
    @use_animation = re ? $1.to_i : 0
  end  
end

#==============================================================================
# ■ RPG::State
#------------------------------------------------------------------------------
# 　ステートのデータクラス。
#==============================================================================

class RPG::State < RPG::BaseItem
  #--------------------------------------------------------------------------
  # ● [追加]:ステートアニメの取得
  #--------------------------------------------------------------------------
  def state_animation
    # キャッシュがある場合、それを返す
    return @state_animation if @state_animation
    # メモ取得
    re = LNX11::RE_STATE_ANIMATION =~ note
    @state_animation = re ? $1.to_i : 0
  end
  #--------------------------------------------------------------------------
  # ● [追加]:ポップアップ表示名の取得
  #--------------------------------------------------------------------------
  def display_name
    # キャッシュがある場合、それを返す
    return @display_name if @display_name
    # メモ取得
    re = LNX11::RE_STATE_DISPLAY =~ note
    @display_name = re ? $1 : name
  end
  #--------------------------------------------------------------------------
  # ● [追加]:ポップアップ非表示の取得
  #--------------------------------------------------------------------------
  def no_display?
    # キャッシュがある場合、それを返す
    return @no_display if @no_display
    # 付加/解除のどちらかで設定されていれば無視する
    if LNX11::RE_STATE_ADD_NO_DISPLAY =~ note ||
       LNX11::RE_STATE_REM_NO_DISPLAY =~ note
      return @no_display = false
    end
    # メモ取得
    re = LNX11::RE_STATE_NO_DISPLAY =~ note
    @no_display = re ? true : false
  end
  #--------------------------------------------------------------------------
  # ● [追加]:付加ポップアップ非表示の取得
  #--------------------------------------------------------------------------
  def add_no_display?
    return true if no_display?
    # キャッシュがある場合、それを返す
    return @add_no_display if @add_no_display
    # メモ取得
    re = LNX11::RE_STATE_ADD_NO_DISPLAY =~ note
    @add_no_display = re ? true : false
  end
  #--------------------------------------------------------------------------
  # ● [追加]:解除ポップアップ非表示の取得
  #--------------------------------------------------------------------------
  def remove_no_display?
    return true if no_display?
    # キャッシュがある場合、それを返す
    return @remove_no_display if @remove_no_display
    # メモ取得
    re = LNX11::RE_STATE_REM_NO_DISPLAY =~ note
    @remove_no_display = re ? true : false
  end
  #--------------------------------------------------------------------------
  # ● [追加]:有利なステートの取得
  #--------------------------------------------------------------------------
  def advantage?
    # キャッシュがある場合、それを返す
    return @advantage if @advantage
    # メモ取得
    re = LNX11::RE_STATE_ADVANTAGE =~ note
    @advantage = re ? true : false
  end
  #--------------------------------------------------------------------------
  # ● [追加]:付加ポップアップタイプの取得
  #--------------------------------------------------------------------------
  def add_popup_type
    # キャッシュがある場合、それを返す
    return @add_popup_type if @add_popup_type != nil
    # メモ取得
    re = LNX11::RE_STATE_ADD_TYPE =~ note
    @add_popup_type = re ? $1.to_i : false
  end
  #--------------------------------------------------------------------------
  # ● [追加]:解除ポップアップタイプの取得
  #--------------------------------------------------------------------------
  def remove_popup_type
    # キャッシュがある場合、それを返す
    return @remove_popup_type if @remove_popup_type != nil
    # メモ取得
    re = LNX11::RE_STATE_REM_TYPE =~ note
    @remove_popup_type = re ? $1.to_i : false
  end
  #--------------------------------------------------------------------------
  # ● [追加]:ポップアップ非表示の取得
  #--------------------------------------------------------------------------
  def no_decoration?
    # キャッシュがある場合、それを返す
    return @no_decoration if @no_decoration
    # 付加/解除のどちらかで設定されていれば無視する
    if LNX11::RE_STATE_ADD_NO_DECORATION =~ note ||
       LNX11::RE_STATE_REM_NO_DECORATION =~ note
      return @no_decoration = false
    end
    # メモ取得
    re = LNX11::RE_STATE_NO_DECORATION =~ note
    @no_decoration = re ? true : false
  end
  #--------------------------------------------------------------------------
  # ● [追加]:付加ポップアップ非表示の取得
  #--------------------------------------------------------------------------
  def add_no_decoration?
    return true if no_decoration?
    # キャッシュがある場合、それを返す
    return @add_no_decoration if @add_no_decoration
    # メモ取得
    re = LNX11::RE_STATE_ADD_NO_DECORATION =~ note
    @add_no_decoration = re ? true : false
  end
  #--------------------------------------------------------------------------
  # ● [追加]:解除ポップアップ非表示の取得
  #--------------------------------------------------------------------------
  def remove_no_decoration?
    return true if no_decoration?
    # キャッシュがある場合、それを返す
    return @remove_no_decoration if @remove_no_decoration
    # メモ取得
    re = LNX11::RE_STATE_REM_NO_DECORATION =~ note
    @remove_no_decoration = re ? true : false
  end
end

#==============================================================================
# ■ Game_Temp
#------------------------------------------------------------------------------
# 　セーブデータに含まれない、一時的なデータを扱うクラスです。このクラスのイン
# スタンスは $game_temp で参照されます。
#==============================================================================

class Game_Temp
  #--------------------------------------------------------------------------
  # ● [追加]:公開インスタンス変数
  #--------------------------------------------------------------------------
  attr_accessor :popup_data            # ポップアップスプライト
  attr_accessor :actor_battler_graphic # アクターのバトラーグラフィック
  attr_accessor :battlelog_clear       # バトルログ消去フラグ
  attr_accessor :last_target_cursor    # 対象選択のカーソル記憶
  #--------------------------------------------------------------------------
  # ● [エイリアス]:オブジェクト初期化
  #--------------------------------------------------------------------------
  alias :lnx11a_initialize :initialize
  def initialize
    # 元のメソッドを呼ぶ
    lnx11a_initialize
    # 追加
    @popup_data = nil
    @actor_battler_graphic = []
    @battlelog_clear = false
    @battle_status_refresh = nil
    clear_last_target_cursor
  end
  #--------------------------------------------------------------------------
  # ● [追加]:バトルステータス更新メソッドの設定
  #--------------------------------------------------------------------------
  def method_battle_status_refresh=(method)
    @battle_status_refresh = method
  end
  #--------------------------------------------------------------------------
  # ● [追加]:バトルステータス更新
  #--------------------------------------------------------------------------
  def battle_status_refresh
    @battle_status_refresh.call if @battle_status_refresh
  end
  #--------------------------------------------------------------------------
  # ● [追加]:対象選択のカーソル記憶をクリア
  #--------------------------------------------------------------------------
  def clear_last_target_cursor
    @last_target_cursor = {:actor => nil, :enemy => nil}
  end
end

#==============================================================================
# ■ Game_ActionResult
#------------------------------------------------------------------------------
# 　戦闘行動の結果を扱うクラスです。属性に関する結果を追加します。
#==============================================================================

class Game_ActionResult
  #--------------------------------------------------------------------------
  # ● 公開インスタンス変数
  #--------------------------------------------------------------------------
  attr_accessor :element_rate               # 属性修正値
  #--------------------------------------------------------------------------
  # ● ダメージ値のクリア
  #--------------------------------------------------------------------------
  alias :lnx11a_clear_damage_values :clear_damage_values
  def clear_damage_values
    # 元のメソッドを呼ぶ
    lnx11a_clear_damage_values
    # 属性修正値をクリア
    @element_rate = 1.0
  end
  #--------------------------------------------------------------------------
  # ● [追加]:属性修正値の取得
  #--------------------------------------------------------------------------
  def element_rate
    return @element_rate if @element_rate
    @element_rate = 1.0
  end
end

#==============================================================================
# ■ Game_BattlerBase
#------------------------------------------------------------------------------
# 　バトラーを扱う基本のクラスです。
#==============================================================================

class Game_BattlerBase
  #--------------------------------------------------------------------------
  # ● [追加]:ポップアップデータの取得
  #--------------------------------------------------------------------------
  def popup_data
    $game_temp.popup_data
  end
  #--------------------------------------------------------------------------
  # ● [エイリアス]:スキル使用コストの支払い
  #--------------------------------------------------------------------------
  alias :lnx11a_pay_skill_cost :pay_skill_cost
  def pay_skill_cost(skill)
    rmp = self.mp
    rtp = self.tp
    # 元のメソッドを呼ぶ    
    lnx11a_pay_skill_cost(skill)
    return unless $game_party.in_battle
    # ポップアップ
    popup_data.popup_regenerate_mp(self, (rmp - self.mp).truncate, true)
    popup_data.popup_regenerate_tp(self, (rtp - self.tp).truncate, true)
  end
end

#==============================================================================
# ■ Game_Battler
#------------------------------------------------------------------------------
# 　スプライトに関するメソッドを追加したバトラーのクラスです。
#==============================================================================

class Game_Battler < Game_BattlerBase
  #--------------------------------------------------------------------------
  # ● 公開インスタンス変数
  #--------------------------------------------------------------------------
  attr_accessor :loop_animation_id     # ループアニメーション ID
  attr_accessor :loop_animation_mirror # ループアニメーション 左右反転フラグ
  #--------------------------------------------------------------------------
  # ● [エイリアス]:スプライトのエフェクトをクリア
  #--------------------------------------------------------------------------
  alias :lnx11a_clear_sprite_effects :clear_sprite_effects
  def clear_sprite_effects
    # 元のメソッドを呼ぶ
    lnx11a_clear_sprite_effects
    @loop_animation_id = 0
    @loop_animation_mirror = false
  end
  #--------------------------------------------------------------------------
  # ● [追加]:ステートのループアニメーションの設定
  #--------------------------------------------------------------------------
  def set_state_animation
    # 表示優先度が高いステートを優先
    sort_states
    anime = @states.collect {|id| $data_states[id].state_animation }
    anime.delete(0)
    @loop_animation_id = anime[0] ? anime[0] : 0
  end
  #--------------------------------------------------------------------------
  # ● [エイリアス]:ステート情報をクリア
  #--------------------------------------------------------------------------
  alias :lnx11a_clear_states :clear_states 
  def clear_states
    # 元のメソッドを呼ぶ
    lnx11a_clear_states
    # ステートアニメ設定
    set_state_animation
  end
  #--------------------------------------------------------------------------
  # ● [エイリアス]:ステートの付加
  #--------------------------------------------------------------------------
  alias :lnx11a_add_state :add_state
  def add_state(state_id)
    # 元のメソッドを呼ぶ
    lnx11a_add_state(state_id)
    # ステートアニメ設定
    set_state_animation
  end
  #--------------------------------------------------------------------------
  # ● [エイリアス]:ステートの解除
  #--------------------------------------------------------------------------
  alias :lnx11a_remove_state :remove_state
  def remove_state(state_id)
    # 元のメソッドを呼ぶ
    lnx11a_remove_state(state_id)
    # ステートアニメ設定
    set_state_animation
  end
  #--------------------------------------------------------------------------
  # ● [追加]:ポップアップディレイの取得
  #--------------------------------------------------------------------------
  def popup_delay
    return @popup_delay if @popup_delay
    popup_delay_clear
  end
  #--------------------------------------------------------------------------
  # ● [追加]:ポップアップディレイの設定
  #--------------------------------------------------------------------------
  def popup_delay=(delay)
    @popup_delay = delay
  end
  #--------------------------------------------------------------------------
  # ● [追加]:ポップアップディレイの設定
  #--------------------------------------------------------------------------
  def popup_delay_clear
    @popup_delay = Array.new(16) { 0 }
  end
  #--------------------------------------------------------------------------
  # ● [エイリアス]:スキル／アイテムの属性修正値を取得
  # 　弱点/耐性の判定を追加します。
  #--------------------------------------------------------------------------
  alias :lnx11a_item_element_rate :item_element_rate
  def item_element_rate(user, item)
    # 元のメソッドを呼ぶ
    rate = lnx11a_item_element_rate(user, item)
    return rate unless $game_party.in_battle
    # レートを結果に保存する
    @result.element_rate = rate
    rate
  end
  #--------------------------------------------------------------------------
  # ● [エイリアス]:ダメージの処理
  # 　吸収による回復のポップアップを生成します。
  #--------------------------------------------------------------------------
  alias :lnx11a_execute_damage :execute_damage
  def execute_damage(user)
    # 元のメソッドを呼ぶ
    lnx11a_execute_damage(user)
    return unless $game_party.in_battle
    # ポップアップ
    popup_data.popup_hp_drain(user, @result.hp_drain)
    popup_data.popup_mp_drain(user, @result.mp_drain)
  end
  #--------------------------------------------------------------------------
  # ● [エイリアス]:HP の再生
  #--------------------------------------------------------------------------
  alias :lnx11a_regenerate_hp :regenerate_hp
  def regenerate_hp
    # 元のメソッドを呼ぶ
    lnx11a_regenerate_hp
    return unless $game_party.in_battle
    # ポップアップ
    popup_data.popup_regenerate_hp(self, @result.hp_damage)
  end
  #--------------------------------------------------------------------------
  # ● [エイリアス]:MP の再生
  #--------------------------------------------------------------------------
  alias :lnx11a_regenerate_mp :regenerate_mp
  def regenerate_mp
    # 元のメソッドを呼ぶ
    lnx11a_regenerate_mp
    return unless $game_party.in_battle
    # ポップアップ
    popup_data.popup_regenerate_mp(self, @result.mp_damage)
  end
  #--------------------------------------------------------------------------
  # ● [エイリアス]:TP の再生
  #--------------------------------------------------------------------------
  alias :lnx11a_regenerate_tp :regenerate_tp
  def regenerate_tp
    rtp = self.tp.to_i
    # 元のメソッドを呼ぶ
    lnx11a_regenerate_tp
    return unless $game_party.in_battle
    # ポップアップ
    popup_data.popup_regenerate_tp(self, rtp - self.tp.to_i)
  end
  #--------------------------------------------------------------------------
  # ● 被ダメージによる TP チャージ
  #--------------------------------------------------------------------------
  alias :lnx11a_charge_tp_by_damage :charge_tp_by_damage
  def charge_tp_by_damage(damage_rate)
    rtp = self.tp.to_i
    # 元のメソッドを呼ぶ
    lnx11a_charge_tp_by_damage(damage_rate)
    return unless $game_party.in_battle
    # ポップアップ
    popup_data.popup_tp_charge(self, rtp - self.tp.to_i)
  end
  #--------------------------------------------------------------------------
  # ● [エイリアス]:スキル／アイテムの使用者側への効果(TP 得)
  #--------------------------------------------------------------------------
  alias :lnx11a_item_user_effect :item_user_effect
  def item_user_effect(user, item)
    rtp = user.tp.to_i
    # 元のメソッドを呼ぶ
    lnx11a_item_user_effect(user, item)
    return unless $game_party.in_battle
    # ポップアップ
    popup_data.popup_tp_gain(user, rtp - user.tp.to_i)
  end
  #--------------------------------------------------------------------------
  # ● [追加]:高さの取得
  #--------------------------------------------------------------------------
  def bitmap_height
    return @bitmap_height if @bitmap_height
    @bitmap_height = 0
  end 
  #--------------------------------------------------------------------------
  # ● [追加]:高さの設定
  #--------------------------------------------------------------------------
  def bitmap_height=(y)
    @bitmap_height = y
  end
  #--------------------------------------------------------------------------
  # ● [追加]:バトル画面 Y 座標(頭上)の取得
  #--------------------------------------------------------------------------
  def screen_y_top
    return screen_y - @bitmap_height
  end
  #--------------------------------------------------------------------------
  # ● [追加]:バトル画面 Y 座標(中心)の取得
  #--------------------------------------------------------------------------
  def screen_y_center
    return screen_y - @bitmap_height / 2
  end
end

#==============================================================================
# ■ Game_Actor
#------------------------------------------------------------------------------
# 　アクターを扱うクラスです。このクラスは Game_Actors クラス（$game_actors）
# の内部で使用され、Game_Party クラス（$game_party）からも参照されます。
#==============================================================================

class Game_Actor < Game_Battler
  #--------------------------------------------------------------------------
  # ● [追加]:公開インスタンス変数
  #--------------------------------------------------------------------------
  attr_accessor :battler_graphic_name    # 後指定のバトラーグラフィック
  attr_accessor :refresh_battler_graphic # バトラーグラフィックの更新フラグ
  attr_accessor :screen_x                # バトル画面 X 座標
  attr_accessor :screen_y                # バトル画面 Y 座標
  attr_accessor :last_actor_command      # 最後に選択したコマンド
  #--------------------------------------------------------------------------
  # ● [再定義]:スプライトを使うか？
  #--------------------------------------------------------------------------
  def use_sprite?
    return true # 使う
  end
  #--------------------------------------------------------------------------
  # ● [追加]:バトル画面 Z 座標の取得
  #--------------------------------------------------------------------------
  def screen_z
    return $game_party.members_screen_z[0] if index == nil
    return $game_party.members_screen_z[index] # Game_EnemyのZ座標は100
  end
  #--------------------------------------------------------------------------
  # ● [エイリアス]:ダメージ効果の実行
  #--------------------------------------------------------------------------
  alias :lnx11a_perform_damage_effect :perform_damage_effect
  def perform_damage_effect
    # 元のメソッドを呼ぶ
    lnx11a_perform_damage_effect
    # シェイクを無効にしている場合、シェイクをクリア
    $game_troop.screen.clear_shake unless LNX11::ACTOR_DAMAGE_SHAKE
  end
  #--------------------------------------------------------------------------
  # ● [再定義]:コラプス効果の実行
  # 　アクターの[特徴>その他>消滅エフェクト]の設定を適用するようにします。
  # 　処理内容は Game_Enemy のものとほぼ同一です。
  #--------------------------------------------------------------------------
  def perform_collapse_effect
    if $game_party.in_battle
      case collapse_type
      when 0
        @sprite_effect_type = :collapse
        Sound.play_enemy_collapse
      when 1
        @sprite_effect_type = :boss_collapse
        Sound.play_boss_collapse1
      when 2
        @sprite_effect_type = :instant_collapse
      end
    end
  end
  #--------------------------------------------------------------------------
  # ● [追加]:グラフィック設定の配列を返す
  #--------------------------------------------------------------------------
  def graphic_name_index
    case LNX11::DEFAULT_BATTLER_GRAPHIC
    when 0 ; [@face_name, @face_index]
    when 1 ; [@character_name, @character_index]
    end
  end
  #--------------------------------------------------------------------------
  # ● [エイリアス]:グラフィックの変更
  #--------------------------------------------------------------------------
  alias :lnx11a_set_graphic :set_graphic
  def set_graphic(*args)
    face = graphic_name_index
    # 元のメソッドを呼ぶ
    lnx11a_set_graphic(*args)
    @refresh_battler_graphic = (face != graphic_name_index)
  end
  #--------------------------------------------------------------------------
  # ● [追加]:デフォルトバトラーグラフィックの取得
  #--------------------------------------------------------------------------
  def facebattler
    $game_temp.actor_battler_graphic[id]
  end
  def facebattler=(bitmap)
    $game_temp.actor_battler_graphic[id] = bitmap
  end
  #--------------------------------------------------------------------------
  # ● [追加]:後指定のバトラーグラフィックファイル名の取得
  #--------------------------------------------------------------------------
  def battler_graphic_name
    return @battler_graphic_name if @battler_graphic_name != nil
    @battler_graphic_name = ""
  end 
  #--------------------------------------------------------------------------
  # ● [追加]:後指定のバトラーグラフィックファイル名の指定
  #--------------------------------------------------------------------------
  def battler_graphic_name=(filename)
    @battler_graphic_name = filename
    @refresh_battler_graphic = true
  end
  #--------------------------------------------------------------------------
  # ● [追加]:顔グラフィックを描画して返す
  # 　処理内容は Window_Base の draw_face に準じたものです。
  #--------------------------------------------------------------------------
  def draw_face(face_name, face_index, enabled = true)
    # サイズ取得
    fw = LNX11::FACE_SIZE[:width]
    fh = LNX11::FACE_SIZE[:height]
    # ビットマップを作成して返す
    bitmap = Cache.face(face_name)
    rect = Rect.new(face_index % 4 * fw, face_index / 4 * fh, fw, fh)
    face = Bitmap.new(fw, fh)
    color = LNX11::FACE_BACK_COLOR
    face.gradient_fill_rect(0, 0, fw, fh, color[0], color[1], true) 
    face.blt(0, 0, bitmap, rect)
    bitmap.dispose
    face
  end
  #--------------------------------------------------------------------------
  # ● [追加]:歩行グラフィックを描画して返す
  # 　処理内容は Window_Base の draw_character に準じたものです。
  #--------------------------------------------------------------------------
  def draw_character(character_name, character_index)
    return unless character_name
    bitmap = Cache.character(character_name)
    sign = character_name[/^[\!\$]./]
    if sign && sign.include?('$')
      cw = bitmap.width / 3
      ch = bitmap.height / 4
    else
      cw = bitmap.width / 12
      ch = bitmap.height / 8
    end
    n = character_index
    src_rect = Rect.new((n%4*3+1)*cw, (n/4*4)*ch, cw, ch)
    character = Bitmap.new(cw, ch)
    character.blt(0, 0, bitmap, src_rect)
    character
  end

  #--------------------------------------------------------------------------
  # ● [追加]:デフォルトバトラーグラフィック設定
  #--------------------------------------------------------------------------
  def default_battler_graphic
    case LNX11::DEFAULT_BATTLER_GRAPHIC
    when 0 # 顔グラフィック
      self.facebattler = draw_face(@face_name, @face_index)
    when 1 # 歩行グラフィック
      self.facebattler = draw_character(@character_name, @character_index)
    end
  end
  #--------------------------------------------------------------------------
  # ● [追加]:バトラーグラフィックの更新
  # 　Sprite_Batter が利用するオブジェクトを更新します。
  #--------------------------------------------------------------------------
  def update_battler_graphic
    @battler_hue = 0
    if !battler_graphic_name.empty?
      # スクリプトで指定されている
      @battler_name = @battler_graphic_name
      dispose_facebattler
    elsif !actor.default_battler_graphic.empty?
      # メモで指定されている
      @battler_name = actor.default_battler_graphic
      dispose_facebattler
    else
      # 何も指定されていない
      @battler_name = ""
      default_battler_graphic
    end
    # 更新したので更新フラグを取り消す
    @refresh_battler_graphic = false
  end
  #--------------------------------------------------------------------------
  # ● [追加]:バトラー用顔グラフィックの解放
  #--------------------------------------------------------------------------
  def dispose_facebattler
    return if self.facebattler == nil
    self.facebattler.dispose
    self.facebattler = nil
  end
  #--------------------------------------------------------------------------
  # ● [エイリアス]:レベルアップメッセージの表示
  #   レベルアップのポップアップを追加します。
  #--------------------------------------------------------------------------
  alias :lnx11a_display_level_up :display_level_up
  def display_level_up(new_skills)
    popup_data.popup_levelup(self)
    lnx11a_display_level_up(new_skills)
  end
end

#==============================================================================
# ■ Game_Party
#------------------------------------------------------------------------------
# 　パーティを扱うクラスです。所持金やアイテムなどの情報が含まれます。このクラ
# スのインスタンスは $game_party で参照されます。
#==============================================================================

class Game_Party < Game_Unit
  #--------------------------------------------------------------------------
  # ● [追加]:公開インスタンス変数
  #--------------------------------------------------------------------------
  attr_accessor :members_screen_x          # バトルメンバー分のX座標配列
  attr_accessor :members_screen_x_nooffset # X:オフセットなし
  attr_accessor :members_screen_y          # バトルメンバー分のY座標配列
  attr_accessor :members_screen_z          # バトルメンバー分のZ座標配列
  #--------------------------------------------------------------------------
  # ● [追加]:バトルメンバーの座標設定
  #--------------------------------------------------------------------------
  def set_members_xyz
    return if members.size == 0
    # 座標をまとめて設定
    set_screen_x
    set_screen_y
    set_screen_z
  end
  #--------------------------------------------------------------------------
  # ● [追加]:バトルメンバーの座標設定
  #--------------------------------------------------------------------------
  def set_members_battle_graphic
    $game_party.battle_members.each {|actor| actor.update_battler_graphic}
  end
  #--------------------------------------------------------------------------
  # ● [追加]:バトル画面 X 座標の設定
  # 　ここで設定した座標は Game_Actor や ステータス表示等で利用されます。
  #--------------------------------------------------------------------------
  def set_screen_x
    @members_screen_x = []
    @members_screen_x_nooffset = []
    padding = LNX11::ACTOR_PADDING[:side]
    if LNX11::ACTOR_CENTERING
      # アクターのセンタリングが有効
      a_spacing = LNX11::ACTOR_SPACING_ADJUST
      padding += (max_battle_members - battle_members.size) * a_spacing
      width = (Graphics.width - padding * 2) / battle_members.size
    else
      # アクターのセンタリングが無効
      width = (Graphics.width - padding * 2) / max_battle_members
    end
    battle_members.each_with_index do |actor, i|
      offset = LNX11::ACTOR_OFFSET[:x]
      @members_screen_x_nooffset[i] = width * i + width / 2 + padding
      @members_screen_x[i] = @members_screen_x_nooffset[i] + offset
      actor.screen_x = @members_screen_x[i]
    end
  end
  #--------------------------------------------------------------------------
  # ● [追加]:バトル画面 Y 座標の設定
  # 　ここで設定した座標は Game_Actor や ステータス表示等で利用されます。
  #--------------------------------------------------------------------------
  def set_screen_y
    offset = LNX11::ACTOR_OFFSET[:y]
    ay = Graphics.height - LNX11::ACTOR_PADDING[:bottom] + offset
    @members_screen_y = Array.new(max_battle_members) {ay}
    battle_members.each {|actor| actor.screen_y = ay}
  end
  #--------------------------------------------------------------------------
  # ● [追加]:バトル画面 Z 座標の設定
  #--------------------------------------------------------------------------
  def set_screen_z
    # 便宜上、XYと同じように配列を作成しておく
    az = LNX11::ACTOR_SCREEN_TONE ? 150 : -10
    @members_screen_z = Array.new(max_battle_members) {az}
  end
  #--------------------------------------------------------------------------
  # ● [オーバーライド]:戦闘開始処理
  #--------------------------------------------------------------------------
  def on_battle_start
    super
    # バトルステータスの更新
    $game_temp.battle_status_refresh
  end
end

#==============================================================================
# ■ Game_Troop
#------------------------------------------------------------------------------
# 　敵グループおよび戦闘に関するデータを扱うクラスです。
#==============================================================================

class Game_Troop < Game_Unit
  #--------------------------------------------------------------------------
  # ● [エイリアス]:セットアップ
  # 　敵グループの座標修正処理を追加します。
  #--------------------------------------------------------------------------
  alias :lnx11a_setup :setup
  def setup(*args)
    # 元のメソッドを呼ぶ
    lnx11a_setup(*args)
    # 敵グループの座標修正
    return unless LNX11::TROOP_X_SCREEN_FIX
    @enemies.each do |enemy|
      # X:解像度がデフォルトでない場合に位置を補正する
      enemy.screen_x *= Graphics.width.to_f / 544
      enemy.screen_x.truncate
      enemy.screen_y += LNX11::TROOP_Y_OFFSET
    end
  end
end

#==============================================================================
# ■ Sprite_Battler
#------------------------------------------------------------------------------
# 　バトラー表示用のスプライトです。
#==============================================================================

class Sprite_Battler < Sprite_Base
  #--------------------------------------------------------------------------
  # ● [オーバーライド]:アニメーションの原点設定
  #--------------------------------------------------------------------------
  def set_animation_origin
    # スーパークラスのメソッドを呼ぶ
    super
    # 画面アニメーションがアクターに再生されたら
    if @animation.position == 3 && @battler != nil && @battler.actor?
      # アニメーションのY座標を修正
      @ani_oy += LNX11::SCREEN_ANIMATION_OFFSET
    end
  end
  #--------------------------------------------------------------------------
  # ● [エイリアス]:転送元ビットマップの更新
  #--------------------------------------------------------------------------
  alias :lnx11a_update_bitmap :update_bitmap
  def update_bitmap
    if @battler.actor? && @battler.refresh_battler_graphic
      # バトラーグラフィックが変更されていれば更新する
      @battler.update_battler_graphic
    end
    if @battler.actor? && @battler.facebattler != nil
      # バトラー用顔グラフィックが作成されていれば、
      # それを Sprite の Bitmap とする
      new_bitmap = @battler.facebattler
      if bitmap != new_bitmap
        self.bitmap = new_bitmap
        init_visibility
      end
    else
      # 元のメソッドを呼ぶ
      lnx11a_update_bitmap
    end
  end
  #--------------------------------------------------------------------------
  # ● [エイリアス]:新しいアニメーションの設定
  #--------------------------------------------------------------------------
  alias :lnx11a_setup_new_animation :setup_new_animation
  def setup_new_animation
    lnx11a_setup_new_animation
    # ループアニメーションの設定
    if @battler.loop_animation_id > 0 && @loop_animation == nil
      animation = $data_animations[@battler.loop_animation_id]
      mirror = @battler.loop_animation_mirror
      start_loop_animation(animation, mirror)
    elsif @battler.loop_animation_id == 0 && @loop_animation
      end_loop_animation
    end
  end
  #--------------------------------------------------------------------------
  # ● [追加]:高さの取得
  #--------------------------------------------------------------------------
  def bitmap_height
    self.bitmap.height
  end
  #--------------------------------------------------------------------------
  # ● [エイリアス]:位置の更新
  #--------------------------------------------------------------------------
  alias :lnx11a_update_position :update_position
  def update_position
    # 元のメソッドを呼ぶ
    lnx11a_update_position
    # 高さを更新
    @battler.bitmap_height = bitmap_height
  end
  if LNX11::ENHANCE_WHITEN
    # 白フラッシュを強めにする
    #------------------------------------------------------------------------
    # ● [再定義]:白フラッシュエフェクトの更新
    #------------------------------------------------------------------------
    def update_whiten
      self.color.set(255, 255, 255, 0)
      self.color.alpha = 192 - (16 - @effect_duration) * 12
    end
  end
  #--------------------------------------------------------------------------
  # ● ループアニメーションの追加
  #--------------------------------------------------------------------------
  #--------------------------------------------------------------------------
  # ● [追加]:クラス変数
  #--------------------------------------------------------------------------
  @@loop_ani_checker = []
  @@loop_ani_spr_checker = []
  #--------------------------------------------------------------------------
  # ● [エイリアス]:オブジェクト初期化
  #--------------------------------------------------------------------------
  alias :lnx11a_initialize :initialize
  def initialize(viewport, battler = nil)
    # 元のメソッドを呼ぶ
    lnx11a_initialize(viewport, battler)
    # ループアニメの残り時間
    @loop_ani_duration = 0
  end
  #--------------------------------------------------------------------------
  # ● [エイリアス]:解放
  #--------------------------------------------------------------------------
  alias :lnx11a_dispose :dispose
  def dispose
    # 元のメソッドを呼ぶ
    lnx11a_dispose
    # ループアニメを解放
    dispose_loop_animation
  end
  #--------------------------------------------------------------------------
  # ● [エイリアス]:フレーム更新
  #--------------------------------------------------------------------------
  alias :lnx11a_update :update
  def update
    # 元のメソッドを呼ぶ
    lnx11a_update
  
    
    update_loop_animation
    @@loop_ani_checker.clear
    @@loop_ani_spr_checker.clear
  end
  #--------------------------------------------------------------------------
  # ● [追加]:ループアニメーション表示中判定
  #--------------------------------------------------------------------------
  def loop_animation?
    @loop_animation != nil
  end
  #--------------------------------------------------------------------------
  # ● [追加]:ループアニメーションの開始
  #--------------------------------------------------------------------------
  def start_loop_animation(animation, mirror = false)
    dispose_loop_animation
    @loop_animation = animation
    if @loop_animation
      @loop_ani_mirror = mirror
      set_loop_animation_rate
      @loop_ani_duration = @loop_animation.frame_max * @loop_ani_rate + 1
      load_loop_animation_bitmap
      make_loop_animation_sprites
      set_loop_animation_origin
    end
  end
  #--------------------------------------------------------------------------
  # ● [追加]:ループアニメーションの速度を設定
  #--------------------------------------------------------------------------
  def set_loop_animation_rate
    @loop_ani_rate = 4
  end
  # LNX09 導入済み
  if true
    #------------------------------------------------------------------------
    # ● [追加]:ループアニメーションの速度を設定
    #------------------------------------------------------------------------
    def set_loop_animation_rate
      default = LNX09::DEFAULT_BATTLE_SPEED_RATE
      if @loop_animation.speed_rate
        @loop_ani_rate = @loop_animation.speed_rate
      else
        @loop_ani_rate = default
      end
    end
  end
  #--------------------------------------------------------------------------
  # ● [追加]:ループアニメーション グラフィックの読み込み
  #--------------------------------------------------------------------------
  def load_loop_animation_bitmap
    animation1_name = @loop_animation.animation1_name
    animation1_hue = @loop_animation.animation1_hue
    animation2_name = @loop_animation.animation2_name
    animation2_hue = @loop_animation.animation2_hue
    @loop_ani_bitmap1 = Cache.animation(animation1_name, animation1_hue)
    @loop_ani_bitmap2 = Cache.animation(animation2_name, animation2_hue)
    if @@_reference_count.include?(@loop_ani_bitmap1)
      @@_reference_count[@loop_ani_bitmap1] += 1
    else
      @@_reference_count[@loop_ani_bitmap1] = 1
    end
    if @@_reference_count.include?(@loop_ani_bitmap2)
      @@_reference_count[@loop_ani_bitmap2] += 1
    else
      @@_reference_count[@loop_ani_bitmap2] = 1
    end
    Graphics.frame_reset
  end
  #--------------------------------------------------------------------------
  # ● [追加]:ループアニメーションスプライトの作成
  #--------------------------------------------------------------------------
  def make_loop_animation_sprites
    @loop_ani_sprites = []
    if @use_sprite && !@@loop_ani_spr_checker.include?(@loop_animation)
      16.times do
        sprite = ::Sprite.new(viewport)
        sprite.visible = false
        @loop_ani_sprites.push(sprite)
      end
      if @loop_animation.position == 3
        @@loop_ani_spr_checker.push(@loop_animation)
      end
    end
    @loop_ani_duplicated = @@loop_ani_checker.include?(@loop_animation)
    if !@loop_ani_duplicated && @loop_animation.position == 3
      @@loop_ani_checker.push(@loop_animation)
    end
  end
  #--------------------------------------------------------------------------
  # ● [追加]:ループアニメーションの原点設定
  #--------------------------------------------------------------------------
  def set_loop_animation_origin
    if @loop_animation.position == 3
      if viewport == nil
        @loop_ani_ox = Graphics.width / 2
        @loop_ani_oy = Graphics.height / 2
      else
        @loop_ani_ox = viewport.rect.width / 2
        @loop_ani_oy = viewport.rect.height / 2
      end
    else
      @loop_ani_ox = x - ox + width / 2
      @loop_ani_oy = y - oy + height / 2
      if @loop_animation.position == 0
        @loop_ani_oy -= height / 2
      elsif @loop_animation.position == 2
        @loop_ani_oy += height / 2
      end
    end
  end
  #--------------------------------------------------------------------------
  # ● [追加]:ループアニメーションの解放
  #--------------------------------------------------------------------------
  def dispose_loop_animation
    if @loop_ani_bitmap1
      @@_reference_count[@loop_ani_bitmap1] -= 1
      if @@_reference_count[@loop_ani_bitmap1] == 0
        @loop_ani_bitmap1.dispose
      end
    end
    if @loop_ani_bitmap2
      @@_reference_count[@loop_ani_bitmap2] -= 1
      if @@_reference_count[@loop_ani_bitmap2] == 0
        @loop_ani_bitmap2.dispose
      end
    end
    if @loop_ani_sprites
      @loop_ani_sprites.each {|sprite| sprite.dispose }
      @loop_ani_sprites = nil
      @loop_animation = nil
    end
    @loop_ani_bitmap1 = nil
    @loop_ani_bitmap2 = nil
  end
  #--------------------------------------------------------------------------
  # ● [追加]:ループアニメーションの更新
  #--------------------------------------------------------------------------
  def update_loop_animation
    return unless loop_animation?
    @loop_ani_duration -= 1
    if @loop_ani_duration % @loop_ani_rate == 0
      if @loop_ani_duration > 0
        frame_index = @loop_animation.frame_max
        frame_index -= (@loop_ani_duration + @loop_ani_rate - 1) / @loop_ani_rate
        loop_animation_set_sprites(@loop_animation.frames[frame_index])
        @loop_animation.timings.each do |timing|
          loop_animation_process_timing(timing) if timing.frame == frame_index
        end
      else
        # 残り時間を再設定してループ
        @loop_ani_duration = @loop_animation.frame_max * @loop_ani_rate + 1
      end
    end
  end
  #--------------------------------------------------------------------------
  # ● [追加]:ループアニメーションの終了
  #--------------------------------------------------------------------------
  def end_loop_animation
    dispose_loop_animation
  end
  #--------------------------------------------------------------------------
  # ● [追加]:ループアニメーションスプライトの設定
  #     frame : フレームデータ（RPG::Animation::Frame）
  #--------------------------------------------------------------------------
  def loop_animation_set_sprites(frame)
    cell_data = frame.cell_data
    @loop_ani_sprites.each_with_index do |sprite, i|
      next unless sprite
      pattern = cell_data[i, 0]
      if !pattern || pattern < 0
        sprite.visible = false
        next
      end
      sprite.bitmap = pattern < 100 ? @loop_ani_bitmap1 : @loop_ani_bitmap2
      sprite.visible = true
      sprite.src_rect.set(pattern % 5 * 192,
        pattern % 100 / 5 * 192, 192, 192)
      if @loop_ani_mirror
        sprite.x = @loop_ani_ox - cell_data[i, 1]
        sprite.y = @loop_ani_oy + cell_data[i, 2]
        sprite.angle = (360 - cell_data[i, 4])
        sprite.mirror = (cell_data[i, 5] == 0)
      else
        sprite.x = @loop_ani_ox + cell_data[i, 1]
        sprite.y = @loop_ani_oy + cell_data[i, 2]
        sprite.angle = cell_data[i, 4]
        sprite.mirror = (cell_data[i, 5] == 1)
      end
      sprite.z = self.z + 316 + i 
      sprite.ox = 96
      sprite.oy = 96
      sprite.zoom_x = cell_data[i, 3] / 100.0
      sprite.zoom_y = cell_data[i, 3] / 100.0
      sprite.opacity = cell_data[i, 6] * self.opacity / 255.0
      sprite.blend_type = cell_data[i, 7]
    end
  end
  #--------------------------------------------------------------------------
  # ● [追加]:SE とフラッシュのタイミング処理
  #     timing : タイミングデータ（RPG::Animation::Timing）
  #--------------------------------------------------------------------------
  def loop_animation_process_timing(timing)
    timing.se.play unless @loop_ani_duplicated
    case timing.flash_scope
    when 1
      self.flash(timing.flash_color, timing.flash_duration * @loop_ani_rate)
    when 2
      if viewport && !@loop_ani_duplicated
        viewport.flash(timing.flash_color, timing.flash_duration * @loop_ani_rate)
      end
    when 3
      self.flash(nil, timing.flash_duration * @loop_ani_rate)
    end
  end
end

#==============================================================================
# ■ Spriteset_Battle
#------------------------------------------------------------------------------
# 　バトル画面のスプライトをまとめたクラスです。
#==============================================================================

class Spriteset_Battle
  #--------------------------------------------------------------------------
  # ● [エイリアス]:オブジェクト初期化
  #--------------------------------------------------------------------------
  alias :lnx11a_initialize :initialize
  def initialize
    # 元のメソッドを呼ぶ
    lnx11a_initialize
    # アクターの背景の作成
    create_actor_background
  end
  #--------------------------------------------------------------------------
  # ● [エイリアス]:解放
  #--------------------------------------------------------------------------
  alias :lnx11a_dispose :dispose
  def dispose
    # 元のメソッドを呼ぶ
    lnx11a_dispose
    # アクターの背景の解放
    dispose_actor_background
  end
  #--------------------------------------------------------------------------
  # ● [追加]:アクターのビューポート
  #--------------------------------------------------------------------------
  def actor_viewport
    LNX11::ACTOR_SCREEN_TONE ? @viewport1 : @viewport2
  end
  #--------------------------------------------------------------------------
  # ● [追加]:アクターの背景の作成
  #--------------------------------------------------------------------------
  def create_actor_background
    return if LNX11::ACTOR_BACKGROUND == 0
    viewport = actor_viewport
    height = LNX11::ACTOR_BACKGROUND_HEIGHT
    case LNX11::ACTOR_BACKGROUND
    when 1
      # グラデーション
      @actor_background = Sprite.new
      back = Bitmap.new(Graphics.width, height)
      color = LNX11::ACTOR_BG_GRADIENT_COLOR
      back.gradient_fill_rect(back.rect, color[0], color[1], true) 
      @actor_background.bitmap = back
    when 2
      # ウィンドウ
      @actor_background = Window_Base.new(0, 0, Graphics.width, height)
    end
    @actor_background.viewport = viewport
    @actor_background.y = Graphics.height - height
    @actor_background.z = viewport == @viewport1 ? 120 : -20
  end
  #--------------------------------------------------------------------------
  # ● [追加]:アクターの背景の解放
  #--------------------------------------------------------------------------
  def dispose_actor_background
    return if LNX11::ACTOR_BACKGROUND == 0
    @actor_background.bitmap.dispose if @actor_background.is_a?(Sprite)
    @actor_background.dispose
  end
  #--------------------------------------------------------------------------
  # ● [再定義]:アクタースプライトの作成
  #--------------------------------------------------------------------------
  def create_actors
    # 座標とグラフィックの設定
    $game_party.set_members_xyz
    $game_party.set_members_battle_graphic
    # スプライトの作成
    viewport = actor_viewport
    @actor_sprites = $game_party.battle_members.collect do |actor|
      Sprite_Battler.new(viewport, actor)
    end
  end
  #--------------------------------------------------------------------------
  # ● [再定義]:アクタースプライトの更新
  #--------------------------------------------------------------------------
  def update_actors
    @actor_sprites.each {|sprite| sprite.update }
  end
end

#==============================================================================
# ■ Scene_Battle
#------------------------------------------------------------------------------
# 　バトル画面の処理を行うクラスです。
#==============================================================================

class Scene_Battle < Scene_Base
  #--------------------------------------------------------------------------
  # ● [エイリアス]:開始処理
  #--------------------------------------------------------------------------
  alias :lnx11a_start :start
  def start
    @last_party_members = party_members
    $game_temp.method_battle_status_refresh = method(:refresh_status)
    $game_temp.clear_last_target_cursor
    $game_party.all_members.each {|actor| actor.last_actor_command = 0 }
    reset_sprite_effects
    standby_message_window_position
    create_targetcursor
    create_popup
    #元のメソッドを呼ぶ
    lnx11a_start
  end
  #--------------------------------------------------------------------------
  # ● [エイリアス]:終了処理
  #--------------------------------------------------------------------------
  alias :lnx11a_terminate :terminate
  def terminate
    dispose_targetcursor
    dispose_popup
    # 元のメソッドを呼ぶ
    lnx11a_terminate
  end
  #--------------------------------------------------------------------------
  # ● [追加]:アクターのスプライト情報の初期化
  #--------------------------------------------------------------------------
  def reset_sprite_effects
    $game_party.battle_members.each do |actor|
      actor.popup_delay_clear
      actor.set_state_animation
    end
  end
  #--------------------------------------------------------------------------
  # ● [エイリアス]:全ウィンドウの作成
  #--------------------------------------------------------------------------
  alias :lnx11a_create_all_windows :create_all_windows
  def create_all_windows
    # 元のメソッドを呼ぶ
    lnx11a_create_all_windows
    create_targethelp_window
  end
  #--------------------------------------------------------------------------
  # ● [エイリアス]:ステータスウィンドウの作成
  #--------------------------------------------------------------------------
  alias :lnx11a_create_status_window :create_status_window
  def create_status_window
    # 元のメソッドを呼ぶ
    lnx11a_create_status_window
    @status_window.set_xy
  end
  #--------------------------------------------------------------------------
  # ● [エイリアス]:情報表示ビューポートの作成
  #--------------------------------------------------------------------------
  alias :lnx11a_create_info_viewport :create_info_viewport
  def create_info_viewport
    # 元のメソッドを呼ぶ
    lnx11a_create_info_viewport
    # ビューポートを修正
    @info_viewport.rect.y = Graphics.height - LNX11::ACTOR_BACKGROUND_HEIGHT
    @info_viewport.rect.height = LNX11::ACTOR_BACKGROUND_HEIGHT
    @status_window.viewport = nil
  end
  #--------------------------------------------------------------------------
  # ● [エイリアス]:パーティコマンドウィンドウの作成
  #--------------------------------------------------------------------------
  alias :lnx11a_create_party_command_window :create_party_command_window
  def create_party_command_window
    # 元のメソッドを呼ぶ
    lnx11a_create_party_command_window
    @party_command_window.viewport = nil
  end
  #--------------------------------------------------------------------------
  # ● [エイリアス]:アクターコマンドウィンドウの作成
  #--------------------------------------------------------------------------
  alias :lnx11a_create_actor_command_window :create_actor_command_window
  def create_actor_command_window
    # 元のメソッドを呼ぶ
    lnx11a_create_actor_command_window
    @actor_command_window.viewport = nil
  end
  #--------------------------------------------------------------------------
  # ● [エイリアス]:ログウィンドウの作成
  #--------------------------------------------------------------------------
  alias :lnx11a_create_log_window :create_log_window
  def create_log_window
    # 元のメソッドを呼ぶ
    lnx11a_create_log_window
    # アニメーションのウェイトのメソッド
    @log_window.method_wait_for_animation = method(:wait_for_animation)
  end  
  #--------------------------------------------------------------------------
  # ● [追加]:ターゲットカーソルの作成
  #--------------------------------------------------------------------------
  def create_targetcursor
    @targetcursor = Sprite_TargetCursor.new
  end
  #--------------------------------------------------------------------------
  # ● [追加]:ターゲットカーソルの解放
  #--------------------------------------------------------------------------
  def dispose_targetcursor
    @targetcursor.dispose
  end
  #--------------------------------------------------------------------------
  # ● [追加]:ターゲットヘルプウィンドウの作成
  #--------------------------------------------------------------------------
  def create_targethelp_window
    @targethelp_window = Window_TargetHelp.new
    @targethelp_window.visible = false
    # ターゲット選択ウィンドウに関連付ける
    @actor_window.help_window = @targethelp_window
    @enemy_window.help_window = @targethelp_window
  end
  #--------------------------------------------------------------------------
  # ● [追加]:ポップアップの作成
  #--------------------------------------------------------------------------
  def create_popup
    $game_temp.popup_data = Popup_Data.new
  end
  #--------------------------------------------------------------------------
  # ● [追加]:ポップアップの解放
  #--------------------------------------------------------------------------
  def dispose_popup
    $game_temp.popup_data.dispose
  end
  #--------------------------------------------------------------------------
  # ● [追加]:ポップアップデータの取得
  #--------------------------------------------------------------------------
  def popup_data
    $game_temp.popup_data
  end
  #--------------------------------------------------------------------------
  # ● [エイリアス]:フレーム更新（基本）
  #--------------------------------------------------------------------------
  alias :lnx11a_update_basic :update_basic
  def update_basic
    # 元のメソッドを呼ぶ
    lnx11a_update_basic
    # 追加したオブジェクトの更新
    update_targetcursor
    update_popup
    refresh_actors
  end
  #--------------------------------------------------------------------------
  # ● [追加]:パーティメンバーの ID 配列の取得
  #--------------------------------------------------------------------------
  def party_members
    $game_party.battle_members.collect {|actor| actor.id }
  end
  #--------------------------------------------------------------------------
  # ● [追加]:メンバーが入れ替わったらオブジェクトを再作成する
  #--------------------------------------------------------------------------
  def refresh_actors
    a_party_members = party_members
    return if @last_party_members == a_party_members
    @last_party_members = a_party_members
    reset_sprite_effects
    @spriteset.dispose_actors
    @spriteset.create_actors
    status_clear
    refresh_status
    Graphics.frame_reset
  end
  #--------------------------------------------------------------------------
  # ● [追加]:ステータスウィンドウの情報を更新(メンバー交代用)
  #--------------------------------------------------------------------------
  def status_clear
    @status_window.all_clear
  end
  #--------------------------------------------------------------------------
  # ● [追加]:メッセージウィンドウの位置の初期化
  # 　毎フレーム呼び出すことで、イベントコマンド以外のメッセージの位置を
  # 固定します。
  #--------------------------------------------------------------------------
  def standby_message_window_position
    $game_message.position = LNX11::MESSAGE_WINDOW_POSITION
  end
  #--------------------------------------------------------------------------
  # ● [再定義]:メッセージウィンドウを開く処理の更新
  #    ステータスウィンドウなどが閉じ終わるまでオープン度を 0 にする。
  #--------------------------------------------------------------------------
  def update_message_open
    if $game_message.busy?
      @party_command_window.close
      @actor_command_window.close
      $game_temp.battlelog_clear = true
    else
      standby_message_window_position
    end
  end
  #--------------------------------------------------------------------------
  # ● [追加]:ターゲットカーソル更新
  #--------------------------------------------------------------------------
  def update_targetcursor
    # 選択中のバトラーの座標を設定する
    if @actor_window.active
      # アクター選択ウィンドウがアクティブ
      @targetcursor.set(@actor_window.actor)
    elsif @enemy_window.active
      # 敵キャラ選択ウィンドウがアクティブ
      @targetcursor.set(@enemy_window.enemy)
    elsif @status_window.index >= 0
      # ステータスウィンドウがアクティブ
      @targetcursor.set(@status_window.actor)
    else
      # どれもアクティブでない場合は非表示
      @targetcursor.hide
    end
    @targetcursor.update
  end

  #--------------------------------------------------------------------------
  # ● [追加]:ポップアップ更新
  #--------------------------------------------------------------------------
  def update_popup
    # ポップアップスプライトの更新
    popup_data.update
  end
  
  #--------------------------------------------------------------------------
  # ● [エイリアス]:パーティコマンド選択の開始
  #--------------------------------------------------------------------------
  alias :lnx11a_start_party_command_selection :start_party_command_selection
  def start_party_command_selection
    # 元のメソッドを呼ぶ
    lnx11a_start_party_command_selection
    # バトルログ削除
    @log_window.lines_clear
    $game_temp.battlelog_clear = LNX11::STORAGE_TURNEND_CLEAR
    # 2ターン目以降のパーテイコマンドスキップ
    return unless @party_command_skip
    @party_command_skip = false
    command_fight if !scene_changing? && @party_command_window.active
  end
  #--------------------------------------------------------------------------
  # ● [エイリアス]:ターン終了
  #--------------------------------------------------------------------------
  alias :lnx11a_turn_end :turn_end
  def turn_end
    @party_command_skip = LNX11::PARTY_COMMAND_SKIP
    # 元のメソッドを呼ぶ
    lnx11a_turn_end
  end
end

#==============================================================================
# ■ LNX11_Window_ActiveVisible
#------------------------------------------------------------------------------
# 　バトル画面でターゲット選択中にウィンドウを非表示にするための
# ウィンドウ用モジュールです。
# ウィンドウの active と visible を連動させる役割があります。
# Window_ActorCommand,Window_BattleSkill, Window_BattleItem で
# インクルードされます。
#==============================================================================

module LNX11_Window_ActiveVisible
  #--------------------------------------------------------------------------
  # ● [オーバーライド]:ウィンドウのアクティブ化
  #--------------------------------------------------------------------------
  def activate
    self.show
    super
  end
  #--------------------------------------------------------------------------
  # ● [オーバーライド]:ウィンドウの非アクティブ化
  #--------------------------------------------------------------------------
  def deactivate
    @help_window != nil ? self.hide : self.visible = false
    super
  end
end

#==============================================================================
# ■ LNX11_Window_FittingList
#------------------------------------------------------------------------------
# 　バトル画面のスキルリスト・アイテムリストを項目数に合わせてリサイズする
# ウィンドウ用モジュールです。
# Window_BattleSkill, Window_BattleItem でインクルードされます。
#==============================================================================

module LNX11_Window_FittingList
  #--------------------------------------------------------------------------
  # ● [追加]:ウィンドウの高さ(最大)
  #--------------------------------------------------------------------------
  def max_height
    @info_viewport.rect.y - @help_window.height
  end
  #--------------------------------------------------------------------------
  # ● [オーバーライド]:リフレッシュ
  #--------------------------------------------------------------------------
  def refresh
    make_item_list
    self.height = [fitting_height(row_max), max_height].min
    super
    self.oy = 0
  end
end
  
#==============================================================================
# ■ LNX11_Window_TargetHelp
#------------------------------------------------------------------------------
# 　バトル画面で、ターゲット選択中にヘルプウィンドウを表示するための
# ウィンドウ用モジュールです。
# Window_BattleActor, Window_BattleEnemy でインクルードされます。
#==============================================================================

module LNX11_Window_TargetHelp
  #--------------------------------------------------------------------------
  # ● [オーバーライド]:ウィンドウの表示
  #--------------------------------------------------------------------------
  def show
    @help_window.show
    super
  end
  #--------------------------------------------------------------------------
  # ● [オーバーライド]:ウィンドウの非表示
  #--------------------------------------------------------------------------
  def hide
    @help_window.hide
    super
  end
end

#==============================================================================
# ■ [追加]:Window_TargetHelp
#------------------------------------------------------------------------------
# 　ターゲットの名前情報やスキルやアイテムの名前を表示します。
#==============================================================================

class Window_TargetHelp < Window_Help
  #--------------------------------------------------------------------------
  # ● [オーバーライド]:オブジェクト初期化
  #--------------------------------------------------------------------------
  def initialize
    super(1) # 1行ヘルプ
  end
  #--------------------------------------------------------------------------
  # ● [オーバーライド]:ウィンドウの非表示
  #--------------------------------------------------------------------------
  def hide
    super
    clear
  end
  #--------------------------------------------------------------------------
  # ● [オーバーライド]:アイテム名設定
  #     item : スキル、アイテム、バトラー等
  #--------------------------------------------------------------------------
  def set_item(item)
    set_text(item ? item : "") # itemの説明ではなく、item自体を渡すようにする
  end
  #--------------------------------------------------------------------------
  # ● [追加]:ゲージ幅
  #--------------------------------------------------------------------------
  def gauge_width
    LNX11::HELP_PARAM_WIDTH
  end
  #--------------------------------------------------------------------------
  # ● [追加]:ゲージ幅(余白を含む)
  #--------------------------------------------------------------------------  
  def gauge_width_spacing
    LNX11::HELP_PARAM_WIDTH + 4
  end
  #--------------------------------------------------------------------------
  # ● [追加]:パラメータエリアの幅
  #--------------------------------------------------------------------------  
  def param_width(size)
    gauge_width_spacing * size
  end
  #--------------------------------------------------------------------------
  # ● [オーバーライド]:リフレッシュ
  #--------------------------------------------------------------------------
  def refresh
    contents.clear
    if @text.is_a?(Game_Battler)
      # バトラー情報の描画
      param = @text.actor? ? LNX11::HELP_ACTOR_PARAM : LNX11::HELP_ENEMY_PARAM
      # ゲージ付きステータス配列
      status = [param[:hp],param[:mp],param[:tp]&&$data_system.opt_display_tp]
      # 名前
      x = contents_width / 2 - contents.text_size(@text.name).width / 2
      name_width = contents.text_size(@text.name).width + 4
      if !status.include?(true)
        # ゲージ付きステータスを描画しない場合
        draw_actor_name(@text, x, 0, name_width)
        x += name_width
        state_width = contents_width - x
      else
        # ゲージ付きステータスを描画する場合
        status.delete(false)
        x -= param_width(status.size) / 2
        draw_actor_name(@text, x, 0, name_width)
        x += name_width
        state_width = contents_width - x - param_width(status.size)
      end
      # ステートアイコン
      if param[:state]
        draw_actor_icons(@text, x, 0, state_width)
      end
      # パラメータの描画
      x = contents_width - param_width(status.size)
      # HP
      if param[:hp]
        draw_actor_hp(@text, x, 0, gauge_width)
        x += gauge_width_spacing
      end
      # MP
      if param[:mp]
        draw_actor_mp(@text, x, 0, gauge_width)
        x += gauge_width_spacing
      end
      # TP
      if param[:tp] && $data_system.opt_display_tp
        draw_actor_tp(@text, x, 0, gauge_width)
        x += gauge_width_spacing
      end
    elsif @text.is_a?(RPG::UsableItem)
      # アイテムかスキルならアイテム名を描画
      draw_item_name_help(item)
    else
      # 通常のテキスト
      super
    end
  end
  #--------------------------------------------------------------------------
  # ● アイテムの描画(中央揃え)
  # 　LNX11a 単体ではターゲットヘルプにアイテム/スキル名が描画されることは
  # ありません。
  #--------------------------------------------------------------------------
  def draw_item_name_help(item)
    w = contents.text_size(@text.name).width
    x = contents_width / 2 - w / 2
    draw_item_name(@text, x - 12, 0, true, w)
  end
end

#==============================================================================
# ■ Window_PartyCommand
#------------------------------------------------------------------------------
# 　バトル画面で、戦うか逃げるかを選択するウィンドウです。
#==============================================================================

class Window_PartyCommand < Window_Command
  #--------------------------------------------------------------------------
  # ● [エイリアス]:オブジェクト初期化
  #--------------------------------------------------------------------------
  alias :lnx11a_initialize :initialize
  def initialize
    # 元のメソッドを呼ぶ
    lnx11a_initialize
    # 座標を設定
    self.x = LNX11::PARTY_COMMAND_RECT[:x]
    self.y = LNX11::PARTY_COMMAND_RECT[:y]
  end
  #--------------------------------------------------------------------------
  # ● [オーバーライド]:ウィンドウ幅の取得
  #--------------------------------------------------------------------------
  def window_width
    return LNX11::PARTY_COMMAND_RECT[:width]
  end
  #--------------------------------------------------------------------------
  # ● [オーバーライド]:表示行数の取得
  #--------------------------------------------------------------------------
  def visible_line_number
    return LNX11::PARTY_COMMAND_VERTICAL ? 2 : 1
  end  
  #--------------------------------------------------------------------------
  # ● [オーバーライド]:桁数の取得
  #--------------------------------------------------------------------------
  def col_max
    return LNX11::PARTY_COMMAND_VERTICAL ? 1 : @list.size
  end
  #--------------------------------------------------------------------------
  # ● [オーバーライド]:アライメントの取得
  #--------------------------------------------------------------------------
  def alignment
    return LNX11::PARTY_COMMAND_ALIGNMENT
  end
  #--------------------------------------------------------------------------
  # ● [オーバーライド]:横に項目が並ぶときの空白の幅を取得
  #--------------------------------------------------------------------------
  def spacing
    return 8
  end
end

#==============================================================================
# ■ Window_ActorCommand
#------------------------------------------------------------------------------
# 　バトル画面で、アクターの行動を選択するウィンドウです。
#==============================================================================

class Window_ActorCommand < Window_Command
  include LNX11_Window_ActiveVisible
  #--------------------------------------------------------------------------
  # ● [オーバーライド]:ウィンドウ幅の取得
  #--------------------------------------------------------------------------
  def window_width
    return LNX11::ACTOR_COMMAND_WIDTH
  end
  #--------------------------------------------------------------------------
  # ● [オーバーライド]:表示行数の取得
  #--------------------------------------------------------------------------
  def visible_line_number
    list_size = LNX11::ACTOR_COMMAND_NOSCROLL ? @list.size : 4
    return LNX11::ACTOR_COMMAND_VERTICAL ? list_size : 1
  end
  #--------------------------------------------------------------------------
  # ● [オーバーライド]:桁数の取得
  #--------------------------------------------------------------------------
  def col_max
    return LNX11::ACTOR_COMMAND_VERTICAL ? 1 : [@list.size, 1].max
  end 
  #--------------------------------------------------------------------------
  # ● [オーバーライド]:アライメントの取得
  #--------------------------------------------------------------------------
  def alignment
    return LNX11::ACTOR_COMMAND_ALIGNMENT
  end
  #--------------------------------------------------------------------------
  # ● [オーバーライド]:横に項目が並ぶときの空白の幅を取得
  #--------------------------------------------------------------------------
  def spacing
    return 8
  end
  #--------------------------------------------------------------------------
  # ● [追加]:X 座標をアクターに合わせる
  #--------------------------------------------------------------------------
  def actor_x(actor)
    ax = $game_party.members_screen_x_nooffset[actor.index] - self.width / 2
    pad = LNX11::STATUS_SIDE_PADDING / 2
    # 画面内に収める
    self.x = [[ax, pad].max, Graphics.width - pad - self.width].min
    self.x += LNX11::ACTOR_COMMAND_OFFSET[:x]
  end
  #--------------------------------------------------------------------------
  # ● [追加]:Y 座標をアクターに合わせる
  #--------------------------------------------------------------------------
  def actor_y(actor)
    self.y = actor.screen_y_top - self.height
    self.y += LNX11::ACTOR_COMMAND_OFFSET[:y]
  end
  #--------------------------------------------------------------------------
  # ● [追加]:固定 Y 座標
  #--------------------------------------------------------------------------
  def screen_y
    if LNX11::ACTOR_COMMAND_Y_POSITION == 0
      self.y = Graphics.height - self.height + LNX11::ACTOR_COMMAND_OFFSET[:y]
    else
      self.y = LNX11::ACTOR_COMMAND_OFFSET[:y]
    end
  end
  #--------------------------------------------------------------------------
  # ● [エイリアス]:セットアップ
  #--------------------------------------------------------------------------
  alias :lnx11a_setup :setup
  def setup(actor)
    # 前のアクターのコマンドを記憶
    @actor.last_actor_command = @list[index] if @actor
    # 元のメソッドを呼ぶ
    lnx11a_setup(actor)
    self.arrows_visible = !LNX11::ACTOR_COMMAND_NOSCROLL
    self.height = window_height
    self.oy = 0
    # アクターコマンドの表示位置で分岐
    case LNX11::ACTOR_COMMAND_POSITION
    when 0
      # アクターの頭上
      actor_x(actor)
      actor_y(actor)
    when 1
      # Y 座標固定
      actor_x(actor)
      screen_y
    when 2
      # XY固定
      self.x = LNX11::ACTOR_COMMAND_OFFSET[:x]
      screen_y
    end
    # 最後に選択したコマンドを選択
    return unless LNX11::LAST_ACTOR_COMMAND
    last_command = @actor.last_actor_command
    if last_command && @list.include?(last_command)
      select(@list.index(last_command))
    end
  end
end

#==============================================================================
# ■ Window_BattleStatus
#------------------------------------------------------------------------------
# 　バトル画面で、パーティメンバーのステータスを表示するウィンドウです。
#==============================================================================

class Window_BattleStatus < Window_Selectable
  #--------------------------------------------------------------------------
  # ● [エイリアス]:オブジェクト初期化
  #--------------------------------------------------------------------------
  alias :lnx11a_initialize :initialize
  def initialize
    @actor_last_status = Array.new($game_party.max_battle_members) { nil }    
    # 元のメソッドを呼ぶ
    lnx11a_initialize
    # ウィンドウを最初から表示
    self.openness = 255
    self.opacity = 0
  end
  #--------------------------------------------------------------------------
  # ● [オーバーライド]:行の高さを取得
  #--------------------------------------------------------------------------
  def line_height
    return LNX11::STATUS_LINE_HEIGHT
  end
  #--------------------------------------------------------------------------
  # ● [オーバーライド]:標準パディングサイズの取得
  #--------------------------------------------------------------------------
  def standard_padding
    return 0
  end
  #--------------------------------------------------------------------------
  # ● [オーバーライド]:表示行数の取得
  #--------------------------------------------------------------------------
  def visible_line_number
    return $data_system.opt_display_tp ? 4 : 3
  end
  #--------------------------------------------------------------------------
  # ● [オーバーライド]:桁数の取得
  #--------------------------------------------------------------------------
  def col_max
    return [item_max, 1].max
  end
  #--------------------------------------------------------------------------
  # ● [オーバーライド]:横に項目が並ぶときの空白の幅を取得
  #--------------------------------------------------------------------------
  def spacing
    return 0
  end
  #--------------------------------------------------------------------------
  # ● [オーバーライド]:項目の高さを取得
  #--------------------------------------------------------------------------
  def item_height
    self.height
  end  
  #--------------------------------------------------------------------------
  # ● [再定義]:ウィンドウ幅の取得
  #--------------------------------------------------------------------------
  def window_width
    Graphics.width
  end
  #--------------------------------------------------------------------------
  # ● [エイリアス]:ウィンドウ高さの取得
  #--------------------------------------------------------------------------
  alias :lnx11a_window_height :window_height
  def window_height
    # 一行目(名前・ステート)の高さを確保する
    lnx11a_window_height - line_height + [24, line_height].max
  end
  #--------------------------------------------------------------------------
  # ● [オーバーライド]:カーソルの更新
  #--------------------------------------------------------------------------
  def update_cursor
    # カーソルを表示しない
    cursor_rect.empty
  end
  #--------------------------------------------------------------------------
  # ● [オーバーライド]:システム色の取得
  # 　ゲージ幅が短すぎる場合、HP,MP,TP の文字を非表示にします。
  #--------------------------------------------------------------------------
  def system_color
    gauge_area_width - @min_offset >= 52 ? super : Color.new
  end  
  #--------------------------------------------------------------------------
  # ● [オーバーライド]:ゲージ背景色の取得
  #   ゲージの透明度を適用します。  
  #--------------------------------------------------------------------------
  def gauge_back_color
    color = super
    color.alpha *= LNX11::STATUS_GAUGE_OPACITY / 255.0
    color
  end
  #--------------------------------------------------------------------------
  # ● [オーバーライド]:ゲージの描画
  #   ゲージの透明度を適用します。
  #--------------------------------------------------------------------------
  def draw_gauge(x, y, width, rate, color1, color2)
    oparate = LNX11::STATUS_GAUGE_OPACITY / 255.0
    color1.alpha *= oparate
    color2.alpha *= oparate
    super
  end
  #--------------------------------------------------------------------------
  # ● [追加]:アクターオブジェクト取得
  #--------------------------------------------------------------------------
  def actor
    $game_party.members[@index]
  end
  #--------------------------------------------------------------------------
  # ● [追加]:座標の設定
  #--------------------------------------------------------------------------
  def set_xy
    # ステータス位置の調整:画面からはみ出ないようにする
    pw = $game_party.members_screen_x.last + LNX11::STATUS_OFFSET[:x]
    pw += gauge_area_width / 2
    right_end = Graphics.width - LNX11::STATUS_SIDE_PADDING
    min_offset = pw > right_end ? pw - right_end : 0
    # ステータスのオフセットを適用
    self.x = LNX11::STATUS_OFFSET[:x] - min_offset
    self.y = LNX11::STATUS_OFFSET[:y] + Graphics.height - self.height
    # ステータス幅の自動調整:位置を調整した分だけ幅を縮める
    @min_offset = LNX11::STATUS_AUTOADJUST ? min_offset : 0
  end
  #--------------------------------------------------------------------------
  # ● [再定義]:ゲージエリアの幅を取得
  #--------------------------------------------------------------------------
  def gauge_area_width
    return LNX11::STATUS_WIDTH
  end
  #--------------------------------------------------------------------------
  # ● [追加]:表示するステート数の取得
  #--------------------------------------------------------------------------
  def states(actor, width)
    icons = (actor.state_icons + actor.buff_icons)[0, width / 24]
    icons.size
  end
  #--------------------------------------------------------------------------
  # ● [追加]:名前とステートの描画(1行表示)
  # 　width の範囲で名前を左揃え、ステートアイコンを右揃えで描画します。
  # ステートを優先して表示するため、アイコンが多すぎて名前の表示領域が
  # 極端に狭くなる場合は名前を描画しません。
  #--------------------------------------------------------------------------
  def draw_actor_name_with_icons(actor, x, y, width = 128, draw_name = true)
    # アイコンのY座標補正
    iy = ([line_height, 24].max - 24) / 2
    # 名前のY座標補正
    ny = ([line_height, 24].max - LNX11::STATUS_NAME_SIZE) / 2
    # 表示するステート数を取得
    icon = states(actor, width)
    if icon > 0
      # 表示するべきステートがある場合
      name_width = width - icon * 24
      ix = x + width - icon * 24
      if name_width >= contents.font.size * 2 && draw_name
        # 名前の表示領域(width) が フォントサイズ * 2 以上なら両方を描画
        draw_actor_name(actor, x, y + ny,  name_width)
        iw = width - name_width
        draw_actor_icons(actor, ix, y + iy, iw)
      else
        # ステートのみ描画
        draw_actor_icons(actor, ix, y + iy, width)
      end
    elsif draw_name
      # ない場合、名前のみ描画
      draw_actor_name(actor, x, y + ny, width)
    end
  end
  #--------------------------------------------------------------------------
  # ● [再定義]:リフレッシュ
  #--------------------------------------------------------------------------
  def refresh
    set_xy
    draw_all_items
  end
  #--------------------------------------------------------------------------
  # ● [オーバーライド]:項目を描画する矩形の取得
  #--------------------------------------------------------------------------
  def item_rect(index)
    rect = super
    rect.x = $game_party.members_screen_x[index] - gauge_area_width / 2
    rect.width = gauge_area_width
    rect
  end
  #--------------------------------------------------------------------------
  # ● [再定義]:基本エリアの矩形を取得
  #--------------------------------------------------------------------------
  def basic_area_rect(index)
    rect = item_rect(index)
    rect
  end
  #--------------------------------------------------------------------------
  # ● [再定義]:ゲージエリアの矩形を取得
  #--------------------------------------------------------------------------
  def gauge_area_rect(index)
    rect = basic_area_rect(index)
    rect.y += [24, line_height].max
    rect.x += @min_offset
    rect
  end
  #--------------------------------------------------------------------------
  # ● [再定義]:基本エリアの描画
  #--------------------------------------------------------------------------
  def draw_basic_area(rect, actor)
    # フォントサイズ変更
    contents.font.size = [LNX11::STATUS_NAME_SIZE, 8].max
    # 名前とステートを描画
    dn = LNX11::STATUS_NAME_SIZE > 0 # 名前を描画するか？
    width = gauge_area_width - @min_offset
    rest  = width % 24
    width += 24 - rest if rest > 0
    draw_actor_name_with_icons(actor, rect.x, rect.y, width, dn)
    # フォントサイズを元に戻す
    reset_font_settings
  end
  #--------------------------------------------------------------------------
  # ● [エイリアス]:ゲージエリアの描画
  #--------------------------------------------------------------------------
  alias :lnx11a_draw_gauge_area :draw_gauge_area
  def draw_gauge_area(*args)
    # フォントサイズ変更
    contents.font.size = LNX11::STATUS_PARAM_SIZE
    # 元のメソッドを呼ぶ
    lnx11a_draw_gauge_area(*args)
    # フォントサイズを元に戻す
    reset_font_settings
  end
  #--------------------------------------------------------------------------
  # ● [再定義]:ゲージエリアの描画（TP あり）
  #--------------------------------------------------------------------------
  def draw_gauge_area_with_tp(rect, actor)
    width = gauge_area_width - @min_offset
    draw_actor_hp(actor, rect.x, rect.y,                   width)
    draw_actor_mp(actor, rect.x, rect.y + line_height,     width)
    draw_actor_tp(actor, rect.x, rect.y + line_height * 2, width)
  end
  #--------------------------------------------------------------------------
  # ● [再定義]:ゲージエリアの描画（TP なし）
  #--------------------------------------------------------------------------
  def draw_gauge_area_without_tp(rect, actor)
    draw_actor_hp(actor, rect.x, rect.y,               gauge_area_width)
    draw_actor_mp(actor, rect.x, rect.y + line_height, gauge_area_width)
  end
  #--------------------------------------------------------------------------
  # ● [再定義]:項目の描画
  #--------------------------------------------------------------------------
  def draw_item(index)
    actor = $game_party.battle_members[index]
    a_status = status(actor)
    # ステータスが変化した場合のみ描画する
    return if @actor_last_status[index] == a_status
    @actor_last_status[index] = a_status
    contents.clear_rect(item_rect(index))
    draw_basic_area(basic_area_rect(index), actor)
    draw_gauge_area(gauge_area_rect(index), actor)
  end
  #--------------------------------------------------------------------------
  # ● [追加]:アクターのステータス配列を返す
  #--------------------------------------------------------------------------
  def status(actor)
    if $data_system.opt_display_tp
      return [actor.name, actor.state_icons + actor.buff_icons,
              actor.mhp, actor.hp, actor.mmp, actor.mp, actor.max_tp, actor.tp]
    else
      # TP を除くステータス配列
      return [actor.name, actor.state_icons + actor.buff_icons,
              actor.mhp, actor.hp, actor.mmp, actor.mp]
    end
  end
  #--------------------------------------------------------------------------
  # ● [追加]:内容の消去
  #--------------------------------------------------------------------------
  def all_clear
    contents.clear
    @actor_last_status = Array.new($game_party.max_battle_members) { nil }    
  end
end

#==============================================================================
# ■ Window_BattleActor
#------------------------------------------------------------------------------
# 　バトル画面で、行動対象のアクターを選択するウィンドウです。
# XPスタイルバトルではバトルステータスを非表示にしないため、
# 選択機能だけを持つ不可視のウィンドウとして扱います。
#==============================================================================

class Window_BattleActor < Window_BattleStatus
  include LNX11_Window_TargetHelp
  #--------------------------------------------------------------------------
  # ● [エイリアス]:オブジェクト初期化
  #--------------------------------------------------------------------------
  alias :lnx11a_wba_initialize :initialize
  def initialize(info_viewport)
    # 元のメソッドを呼ぶ
    lnx11a_wba_initialize(info_viewport)
    # ウィンドウを画面外に移動
    self.y = Graphics.height
  end
  #--------------------------------------------------------------------------
  # ● [オーバーライド]:ヘルプテキスト更新
  #--------------------------------------------------------------------------
  def update_help
    @help_window.set_item(actor)
  end
  #--------------------------------------------------------------------------
  # ● [オーバーライド]:ウィンドウ幅の取得
  #--------------------------------------------------------------------------
  def window_width
    64
  end
  #--------------------------------------------------------------------------
  # ● [オーバーライド]:全項目の描画
  #--------------------------------------------------------------------------
  def draw_all_items
    # 何もしない    
  end
  #--------------------------------------------------------------------------
  # ● [オーバーライド]:項目の描画
  #--------------------------------------------------------------------------
  def draw_item(index)
    # 何もしない
  end
  #--------------------------------------------------------------------------
  # ● [エイリアス]:ウィンドウの表示
  #--------------------------------------------------------------------------
  alias :lnx11a_show :show
  def show
    # 元のメソッドを呼ぶ
    lnx11a_show
    return self unless LNX11::LAST_TARGET
    # 最後に選択したアクターを選択
    last_target = $game_temp.last_target_cursor[:actor]
    if last_target && $game_party.members.include?(last_target)
      select($game_party.members.index(last_target))
    end
    self
  end
  #--------------------------------------------------------------------------
  # ● [エイリアス]:ウィンドウの非表示
  #--------------------------------------------------------------------------
  alias :lnx11a_hide :hide
  def hide
    # 元のメソッドを呼ぶ
    lnx11a_hide
    # 選択したアクターを記憶
    $game_temp.last_target_cursor[:actor] = actor
    self
  end
end

#==============================================================================
# ■ Window_BattleEnemy
#------------------------------------------------------------------------------
# 　バトル画面で、行動対象の敵キャラを選択するウィンドウです。
# 横並びの不可視のウィンドウとして扱います。
#==============================================================================

class Window_BattleEnemy < Window_Selectable
  include LNX11_Window_TargetHelp
  #--------------------------------------------------------------------------
  # ● [エイリアス]:オブジェクト初期化
  #--------------------------------------------------------------------------
  alias :lnx11a_initialize :initialize
  def initialize(info_viewport)
    # 元のメソッドを呼ぶ
    lnx11a_initialize(info_viewport)
    # ウィンドウを画面外に移動
    self.y = Graphics.height
  end
  #--------------------------------------------------------------------------
  # ● [オーバーライド]:ウィンドウ幅の取得
  #--------------------------------------------------------------------------
  def window_width
    64
  end
  #--------------------------------------------------------------------------
  # ● [オーバーライド]:指定行数に適合するウィンドウの高さを計算
  #--------------------------------------------------------------------------
  def fitting_height(line_number)
    super(1)
  end
  #--------------------------------------------------------------------------
  # ● [オーバーライド]:桁数の取得
  #--------------------------------------------------------------------------
  def col_max
    return item_max
  end
  #--------------------------------------------------------------------------
  # ● [オーバーライド]:ヘルプテキスト更新
  #--------------------------------------------------------------------------
  def update_help
    @help_window.set_item(enemy)
  end
  #--------------------------------------------------------------------------
  # ● [エイリアス]:ウィンドウの表示
  #--------------------------------------------------------------------------
  alias :lnx11a_show :show
  def show
    # 元のメソッドを呼ぶ
    lnx11a_show
    return self unless LNX11::LAST_TARGET
    # 最後に選択した敵キャラを選択
    last_target = $game_temp.last_target_cursor[:enemy]
    if last_target && $game_troop.alive_members.include?(last_target)
      select($game_troop.alive_members.index(last_target))
    end
    self
  end
  #--------------------------------------------------------------------------
  # ● [エイリアス]:ウィンドウの非表示
  #--------------------------------------------------------------------------
  alias :lnx11a_hide :hide
  def hide
    # 元のメソッドを呼ぶ
    lnx11a_hide
    # 選択した敵キャラを記憶
    $game_temp.last_target_cursor[:enemy] = enemy
    self
  end
end

#==============================================================================
# ■ Window_BattleSkill
#------------------------------------------------------------------------------
# 　いくつかのモジュールをインクルードします。
#==============================================================================

class Window_BattleSkill < Window_SkillList
  include LNX11_Window_ActiveVisible
  include LNX11_Window_FittingList if LNX11::FITTING_LIST
end

#==============================================================================
# ■ Window_BattleItem
#------------------------------------------------------------------------------
# 　いくつかのモジュールをインクルードします。
#==============================================================================

class Window_BattleItem < Window_ItemList
  include LNX11_Window_ActiveVisible
  include LNX11_Window_FittingList if LNX11::FITTING_LIST
end
